/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                        OS_Z180.h                         *
*        Hitachi 64180 / Zilog Z-8018x Specific Code       *
*                   BANKING MEMORY MODEL                   *
***********************************************************/
#ifndef _OS_Z180_
 #define _OS_Z180_

/*
************************************************************
*                     Z-180 Specific code
*                         (Banked)
*                       MISCELLANEOUS
************************************************************
*/
#define  RST_00H              0xC7   /* Restart instructions */
#define  RST_08H              0xCF
#define  RST_10H              0xD7
#define  RST_18H              0xDF
#define  RST_20H              0xE7
#define  RST_28H              0xEF
#define  RST_30H              0xF7
#define  RST_38H              0xFF
/*
************************************************************
*               MACROS (compiler spezific)
************************************************************
*/
 #define  OS_CRITICAL_SR       0                             // do not save the CPU-register during INT-disable

 #define  OS_ENTER_CRITICAL()  disable_interrupt()
 #define  OS_EXIT_CRITICAL()   enable_interrupt()
 #define  OS_TASK_SW()         _opc(RST_20H)   
                         /* Execute a RESTART instruction */
/*
************************************************************
*                    ENDIAN CONVENTION
************************************************************
*/
 #define  LITTLE_ENDIAN   3412
 #define  BIG_ENDIAN      1234

 #define  BYTE_ORDER  LITTLE_ENDIAN   /* endianess of CPU */

//------------ for little-endian protocols -----------------
 #if BYTE_ORDER == LITTLE_ENDIAN                           // for example: USB 
  #define L_htonl(n)       (n)
  #define L_htons(n)       (n)
 #else
  #define L_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define L_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  L_ntohl          L_htonl
 #define  L_ntohs          L_htons
//-------------- for big-endian protocols ------------------
 #if BYTE_ORDER == BIG_ENDIAN                              // for example: 802.3 , TCP/IP
  #define B_htonl(n)       (n)
  #define B_htons(n)       (n)
 #else
  #define B_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define B_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  B_ntohl          B_htonl
 #define  B_ntohs          B_htons
/*
************************************************************
*                 POINTER AND REENTRANCE
************************************************************
*/
 #define  OS_EXT  extern   /* define external declaration */
 #define  OS_FAR              /* not needed               */
 #define  OS_HUGE far         /* define OS_HUGE           */
 #define  OS_CONST const      /* define as const          */
 #define  OS_ROMDATA          /* area locations in ROM    */
 #define  KC51_REENTRANT                    /* not needed */
/*
************************************************************
*                      DATA TYPES
************************************************************
*/
 typedef unsigned char  BOOLEAN;
 typedef unsigned char  U08;
 typedef signed   char  S08;
 typedef unsigned int   U16;
 typedef signed   int   S16;
 typedef unsigned long  U32;
 typedef signed   long  S32;
 typedef float          FP;
 typedef U08            OS_STK_TYPE;

/*
************************************************************
*                       CONSTANTS
************************************************************
*/
 #ifndef FALSE
  #define FALSE  0
 #endif
 #ifndef TRUE
  #define TRUE   1
 #endif
 #ifndef NULL
  #define NULL   ((void *)0)
 #endif

/*
************************************************************
*                  COMPILING DEFINITIONS
************************************************************
*/
#ifndef OS_PACKED
 #define OS_PACKED
#endif
#ifndef OS_PACKED_ATTR
 #define OS_PACKED_ATTR
#endif
#ifndef OS_ALIGN_ATTR
 #define OS_ALIGN_ATTR
#endif

/************************** END ***************************/
#endif
