/*************************************************************
*
* Description:
*	Am188ES - Example program using Microsoft C 8.00 and Paradigm Locate
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define	TASK_STK_SIZE   512

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];

OS_MUX Mutex1;
OS_EVG Events1;

void far  Task1(void *Tdata);
void far  Task2(void *Tdata);
void far  Task3(void *Tdata);

void main()
{
    long far *irqAddr;

    OS_Init();

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);

    irqAddr=(long far *) FARPTR(0,0x0048);      // use Timer-1 as soft-INT
    *irqAddr = (long) &OSCtxSw;                 // for context-switch

    OS_Start();
}

void AppTickInit(void)
{
    long far *irqAddr;
    
    irqAddr=(long far *) FARPTR(0,0x004C);      // use Timer-0
    *irqAddr = (long) &OSTickISR;               // for timer-ISR
    _asm {
        cli
        push    dx
        mov     dx,0FF28h                       ; IMASK (see REG186.inc)
        in      ax,dx                           ; get IMASK
        and     ax,0000011111111100b            ; enable Timer
        out     dx,ax
        pop     dx
        sti
    }
}

void far Task1(void *Tdata)
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    Tdata = Tdata;
    i=0;
    AppTickInit();
    ptr=NULL;
    while(1) {
        i=0;
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(10);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(10);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}


void far Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // suspending
        if(!returnOk) {
            i=0;
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(10);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
		}
    }
}

void far Task3(void *Tdata)
{
    U32         i;

    Tdata = Tdata;
    i=0;
    while(1) {
        OS_TimeDly(10);                                 // for Task2
        i++;
    }
}
