;****************************************************************
;                        OS_BOOT186.ASM
;            80186/80188 Real Time Operating System
;                     Am188ES startup-code
;****************************************************************

        .186

        NAME    OS_BOOT186
        TITLE   BOOT Startup Code for MASM 6.0

        include ..\inc\Reg186.inc

COPY_SEG segment para public 'COPY'
    COPYRIGHT db 0FFh,'pC/OS V1.40a'
;                 0    123456789ABC
COPY_SEG ends

_text   segment para public 'CODE'
_text   ends
_data   segment para public 'DATA'
_data   ends
DGROUP  group _data
_bss    segment word public 'BSS'
_bss    ends
DGROUP  group _bss
_stack  segment para stack 'STACK'
_stack  ends
DGROUP  group _stack

;******************* External references *************************************

        EXTRN   C       main : far             	; User application entry point
        _main   equ     main

;********************** must set for call a C-function ***********************

public  __acrtused                      ; Object modules reference this symbol
        __acrtused = 9876h              ; to pull in startup code
public  __aDBused                       ; Created when the /qc option is used
        __aDBused = 0d6d6h

;**************************** Startup-Code ***********************************
_text   segment
        assume cs:_text, ds:_data

_startup        PROC    far             ; startup code entry point
        cli                             ; clear INT
        cld                             ; clear direction to forward
        lea     si, offset BOOTINI      ; get begin of table
IniLoop:
        mov     dx,cs:[si]              ; get register
        cmp     dx,0FFFFh               ; end of Table?
        jz      EndIni
        add     si,2                    ; pointer to contens
        mov     ax,cs:[si]              ; get contens
        out     dx,ax                   ; save contens to register
        add     si,2                    ; pointer to next register
        jmp     short IniLoop           ; to next register in table

;***************************************************************************
; Init table for Am188ES
;***************************************************************************
BOOTINI:
                dw      RELREG,    0000000011111111b    ; peripherial control block register (at FFxxh)
;               dw      RESCON,    0000000000000000b    ; Reset configuration register (read only)
;               dw      PRL,       0000000000000000b    ; Processor Release Level Register (read only)
                dw      AUXCON,    0000000000000000b    ; Auxiliary config register (CTS0/RTR0 CTS1/RTR1)
                dw      SYSCON,    0000000000000000b    ; System configuration register (noPSM noDSM CLKA)
                dw      UMCS,      1000000010111100b    ; upper memory chip select register (fROM 512k noWait noRDY)
                dw      LMCS,      0111111110111100b    ; lower memory chip select register (sRAM 512k noWait noRDY)
                dw      MMCS,      0000000111111100b    ; Midrange memory chip select register (no used)
                dw      MPCS,      1000000100111111b    ; PCS/MCS auxiliary register
                dw      PACS,      PACSi                ; Peripherial chip select register (IO-Devices begin 0000h)
                dw      MDRAM,     0000000000000000b    ; Memory partition register (not used)
                dw      CDRAM,     0000000000010010b    ; Clock prescaler register (not used)
                dw      EDRAM,     0000000000000000b    ; Enable rcu register (not used)
                dw      WDTCON,    3333h                ; sequenz  to open WDTCON
                dw      WDTCON,    0CCCCh               ; -"-
                dw      WDTCON,    0100000010000000b    ; Watchdog control register (diable - 1.677sec. TimeOut)
                dw      I0CON,     0000000000010001b    ; INT-0 control register (enable level-triggert prio-01 for ESCC)
                dw      I1CON,     0000000000010101b    ; INT-1 control register (enable level-triggert prio-05 for IPAC)
                dw      I2CON,     0000000000001111b    ; INT-2 control register (disable low-pri)
                dw      I3CON,     0000000000001111b    ; INT-3 control register (disable low-pri)
                dw      I4CON,     0000000000001111b    ; INT-4 control register (disable low-pri)
                dw      DMA0CON,   0000000000001111b    ; DMA-0 / INT-5 interrupt control register (disable low-pri)
                dw      DMA1CON,   0000000000001111b    ; DMA-1 / INT-6 interrupt control register (disable low-pri)
                dw      SP0CON,    0000000000011010b    ; Serial Port-0 interrupt control register (disable prio-02 - RS232)
                dw      SP1CON,    0000000000011011b    ; Serial Port-1 interrupt control register (disable prio-03 - RS485)
                dw      INTSTS,    0000000000000000b    ; Interrupt Status register (clear)
;               dw      REQST,     0000000000000000b    ; Interrupt Request register (read only)
                dw      INSERV,    0000000000000000b    ; Interrupt in service register (clear)
                dw      PRIMSK,    0000000000000111b    ; Priority mask register (all priorities)
                dw      IMASK,     0000011111111101b    ; Interrupt mask register (default all disable)
;               dw      POLLST,    0000000000000000b    ; Poll status register (read only)
;               dw      POLL,      0000000000000000b    ; Poll register (read only)
                dw      EOI,       1000000000000000b    ; End-of-Interrupt register (clear all)
                dw      T0CON,     0100000000000000b    ; Timer-0 mode and control register (disable)
                dw      T1CON,     0100000000000000b    ; Timer-1 mode and control register (disable)
                dw      T2CON,     1110000000001001b    ; Timer-2 mode and control register (enable/INT/cont)
                dw      T0CNT,     0000000000000000b    ; Timer-0 count register
                dw      T1CNT,     0000000000000000b    ; Timer-1 count register
                dw      T2CNT,     0000000000000000b    ; Timer-2 count register
                dw      T0CMPA,    0000000000000000b    ; Timer-0 maxcount compare register A 
                dw      T0CMPB,    0000000000000000b    ; Timer-0 maxcount compare register B
                dw      T1CMPA,    0000000000000000b    ; Timer-1 maxcount compare register A
                dw      T1CMPB,    0000000000000000b    ; Timer-1 maxcount compare register B
                dw      T2CMPA,    1100000000000000b    ; Timer-2 maxcount compare register A (5ms for 39.3216MHz)
                dw      TCUCON,    0000000000000111b    ; Timer interrupt control register (enable low-pri)
                dw      D0CON,     0000000000000100b    ; DMA-0 control register
                dw      D1CON,     0000000000000100b    ; DMA-1 control register
                dw      D0TC,      0000000000000000b    ; DMA-0 transfer count register
                dw      D1TC,      0000000000000000b    ; DMA-1 transfer count register
                dw      D0DSTH,    0000000000000000b    ; DMA-0 destination address high register
                dw      D1DSTH,    0000000000000000b    ; DMA-1 destination address high register
                dw      D0DSTL,    0000000000000000b    ; DMA-0 destination address low register
                dw      D1DSTL,    0000000000000000b    ; DMA-1 destination address low register
                dw      D0SRCH,    0000000000000000b    ; DMA-0 source address high register
                dw      D1SRCH,    0000000000000000b    ; DMA-1 source address high register
                dw      D0SRCL,    0000000000000000b    ; DMA-0 source address low register
                dw      D1SRCL,    0000000000000000b    ; DMA-1 source address low register
                dw      SP0CT,     0000000000000000b    ; Serial port-0 control register (default disable)
                dw      SP1CT,     0000000000000000b    ; Serial port-1 control register (default disable)
                dw      SP0STS,    0000000000000000b    ; Serial port-0 status register (clear)
                dw      SP1STS,    0000000000000000b    ; Serial port-1 status register (clear)
;               dw      SP0TD,     0000000000000000b    ; Serial port-0 trasmit register (not to init)
;               dw      SP1TD,     0000000000000000b    ; Serial port-1 trasmit register (not to init)
;               dw      SP0RD,     0000000000000000b    ; Serial port-0 receive register (read only)
;               dw      SP1RD,     0000000000000000b    ; Serial port-1 receive register (read only)
                dw      SP0BAUD,   0000000010000000b    ; Serial port-0 baud rate divisor register (19200Bd)
                dw      SP1BAUD,   0000000010000000b    ; Serial port-1 baud rate divisor register (19200Bd)
                dw      PDATA0,    0100001110100100b    ; PIO-0 data register (clear and set Bus-Pin's Low)
                dw      PDIR0,     1010000001100100b    ; PIO-0 direction register
                dw      PIOMODE0,  1101110001011011b    ; PIO-0 mode register (Bus control + isp)
                dw      PDATA1,    1111111111111111b    ; PIO-1 data register (clear)
                dw      PDIR1,     1000001000000000b    ; PIO-1 direction register
                dw      PIOMODE1,  0100000000000000b    ; PIO-1 mode register (RS232 + RS485)
                dw      0FFFFh,0FFFFh                   ; End of table
;***************************************************************************

ISRD:   push    ax							   ; Dummy-ISR
        push    dx
        mov     dx,EOI
        mov     ax,8000h
        out     dx,ax
        pop     dx
        pop     ax
        iret

EndIni:	mov     cx,21                           ; for 21 table entries
        xor     ax,ax                           ; as segment 0000h
        mov     es,ax                           ; in es
        mov     di,ax                           ; vector-table begins at 0000:0000h
ISRi:   lea     ax, offset ISRD                 ; get ISR-Vector for Dummy-ISR
        mov     word ptr es:[di],ax             ; save in Vector table (offset)
        mov     ax,cs
        mov     word ptr es:[di+2],ax           ; save in Vector table (segment)
        add     di,4                            ; to next entry
        loop    short ISRi                      ; loop
;
; Initialize the stack segment and pointer registers to point to the
; default stack.
;
        mov     ss, cs:_dataseg
        mov     sp, offset DGROUP:__tos
        assume  ss:DGROUP

;
; Copy DGROUP initialized data from its position in ROM to the target
; address in RAM.  Because this is a group, there is never more than
; 64K of data to copy.
;
;
;       mov     si, offset brdata
;       mov     di, offset DGROUP:idata
;       mov     cx, offset DGROUP:bdata
;       sub     cx, di
;       jcxz    @F
;       shr     cx, 1
;       rep     movsw
;@@:
;
; Zero out the BSS area
;
        mov     es, cs:_dataseg
        assume  es:DGROUP
        xor     ax, ax
        mov     di, offset DGROUP:bdata
        mov     cx, offset DGROUP:__edata
        sub     cx, di
        jcxz    @F
        shr     cx, 1
        rep     stosw
@@:
;
; Setup the segment register conventions that will be used to call the
; Microsoft C main() function.
;
        mov     ds, cs:_dataseg
        mov     es, cs:_dataseg
        assume  ds:DGROUP, es:DGROUP

;
; Call the C entry point main() - initialization is complete and user
; application can take control.
;
        sti
        call    main
        nop
;
; A return from main() is application dependent.  If PDREMOTE is being used,
; we return the contents of AX as the application exit code; otherwise,
; the user must decide how to handle this condition.
;
        jmp     _startup
_startup        ENDP

_dataseg dw DGROUP              ; Used to access DGROUP value

_text   ends

_data   segment
public  idata
idata   label   byte                    ; Mark the start of the DATA class
null    dw      007ffh                  ; NULL address (within the segment)
_data   ends

_bss    segment
public  bdata
bdata   label   byte                    ; Mark the start of the BSS class
_bss    ends


;
; The stack segment differs from the Microsoft DOS startup code in that
; the stack is assigned a fixed length.
;
_stack  segment
public __edata
public __tos
__edata label word
db      0400h dup (?)			; system-stack (kernel or linear-prog.)
__tos label word                        ; Mark the top of stack
_stack  ends

                END     _startup
