;****************************************************************
;*                       pC/OS 1.68a                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                       OS_186_a.asm                           *
;*           8051 Specific Code (LARGE MEMORY MODEL)            *
;*                        using KEIL                            *
;****************************************************************
$include(Reg51.inc)                  ; from Keil-C51

;CPU specific definitions 
C51_CRYSTAL_FREQ   EQU 3571200
OS_TICKS_PER_SEC   EQU 10

;Determine the reload values for timer 0 this is automated by the next macro
T0_RELOAD  EQU  C51_CRYSTAL_FREQ / (OS_TICKS_PER_SEC * 6)
T0_RELOADH EQU  (65536 - (T0_RELOAD AND 65535)) SHR 8
T0_RELOADL EQU  (65536 - (T0_RELOAD AND 65535)) AND 255


        PUBLIC  OSStartHighRdy
        PUBLIC  OSCtxSw
        PUBLIC  OSIntCtxSw
        PUBLIC  OSTickISR

        EXTRN   CODE (OSIntEnter)
        EXTRN   CODE (OSIntExit)
        EXTRN   CODE (OSSystemTick)
        EXTRN   XDATA (OSTCBCur)
        EXTRN   XDATA (OSTCBHighRdy)
        EXTRN   XDATA (OSIntNesting)

?STACK  SEGMENT IDATA
        RSEG    ?STACK

OSStack:
        ds      60h
OSStkStart      IDATA OSStack

OSFrameSize     equ     18

;****************************************************************
; The following macro is the assembly code to save the current 
; task context
save_context    MACRO
        PUSH    PSW
        PUSH    ACC
        PUSH    B
        PUSH    0
        PUSH    1
        PUSH    2
        PUSH    3
        PUSH    4
        PUSH    5
        PUSH    6
        PUSH    7
        PUSH    DPH
        PUSH    DPL
        mov     a,08h
        push    acc
        mov     a,09h
        push    acc
        mov     a,SP
        inc     acc
        PUSH    acc              ;save the corrected stackpointer
ENDM

;****************************************************************
; The following macro is the assembly code to restore the current 
; task context
restore_context MACRO
        POP     ACC             ;discard SP
        pop     acc
        mov     09h,a
        pop     acc
        mov     08h,a
        POP     DPL
        POP     DPH
        POP     7
        POP     6
        POP     5
        POP     4
        POP     3
        POP     2
        POP     1
        POP     0
        POP     B
        POP     ACC
        POP     PSW
        RETI
ENDM

;****************************************************************
; The following macro is the assembly code to save the current 
; stack to xdata        DPTR = begin of user stack-1
save_stack      MACRO
        LOCAL   save_ctx
        CLR     EA
        CLR     C
        MOV     A,SP
        SUBB    A,#OSStkStart
        MOV     R5,A            ;R5 = bytes to copy
        MOV     R4,A            ;R4 = count
        MOV     A,#(OSStkStart-1) ;copy R5 bytes from *stk to OSStack
        MOV     R1,A
save_ctx:
        INC     R1
        INC     DPTR
        MOV     A,@R1
        MOVX    @DPTR,A
        DJNZ    R4,save_ctx

        MOV     A,R1
        CLR     C
        SUBB    A,#OSStkStart
        CLR     C
        INC     DPTR
        ADD     A,#2            ;correct SP for self and zero-pos
        MOVX    @DPTR,A         ;fix SP in user stack
        MOV     R0,DPH
        MOV     R1,DPL          ;R0,R1 = *vstk
ENDM

;****************************************************************
; The following macro is the assembly code to restore the current 
; stack from xdata      DPTR = begin of user stack-1
restore_stack   MACRO
        LOCAL   rest_ctx
        CLR     EA
        INC     DPTR            ;over segment-type
        MOVX    A,@DPTR         ;get user-stack 
        MOV     R0,A
        INC     DPTR
        MOVX    A,@DPTR
        MOV     R1,A
        MOV     DPH,R0          ;DPTR = user-stack
        MOV     DPL,R1
        MOVX    A,@DPTR         ;get bytes to copy (= SP - #OSStkStart)
        MOV     R5,A            ;R5 = bytes to copy
        MOV     R4,A            ;R4 = count
        MOV     A,DPL
        CLR     C
        SUBB    A,R5
        MOV     R3,A
        MOV     A,DPH
        SUBB    A,#0
        MOV     R2,A            ;R2,3 = srcPTR-1
        MOV     DPH,R2
        MOV     DPL,R3          ;DPTR = srcPTR-1
        MOV     A,#(OSStkStart-1) ;copy R5 bytes from OSStack to *stk
        MOV     R1,A
rest_ctx: 
        INC     DPTR
        INC     R1
        MOVX    A,@DPTR
        MOV     @R1,A
        DJNZ    R4,rest_ctx

        MOV     SP,R1           ;now R1 is the new SP

        SETB    EA
ENDM


; Vector Hardware Interrupts
;
INTVECT_LOC     EQU      0000h          ; INT-vectortable location

        CSEG    AT INTVECT_LOC+000Bh    ; TF0
        ljmp    OSTickISR


PCOS51A_CODE_PR SEGMENT	CODE
        RSEG    PCOS51A_CODE_PR

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
OSStartHighRdy  PROC
        CLR     EA
        MOV     DPTR,#(OSTCBHighRdy)
        INC     DPTR            ;over segment-type
        MOVX    A,@DPTR
        MOV     R0,A
        INC     DPTR
        MOVX    A,@DPTR
        MOV     R1,A            ;R0:R1  = OSTCBHighRdy
        MOV     DPTR,#(OSTCBCur)
        INC     DPTR            ;over segment-type
        MOV     A,R0            
        MOVX    @DPTR,A
        INC     DPTR
        MOV     A,R1
        MOVX    @DPTR,A         ;OSTCBCur = OSTCBHighRdy
        MOV     DPL,R1
        MOV     DPH,R0          ;DPTR = pointer to OSTCBCur->OSTCBStkPtr
        restore_stack 
        restore_context
ENDP

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
OSCtxSw         PROC
        save_context
ctxSwI:
        MOV     DPTR,#(OSTCBCur);stave stack ptr in TCB
        INC     DPTR            ;over segment-type
        MOVX    A,@DPTR         ;ptr = OSTCBCur-StkPtr
        MOV     R0,A
        INC     DPTR
        MOVX    A,@DPTR
        MOV     R1,A
        MOV     DPL,R1
        MOV     DPH,R0
        INC     DPTR            ;over segment-type
        MOVX    A,@DPTR
        MOV     R0,A
        INC     DPTR
        MOVX    A,@DPTR
        MOV     R1,A
        MOV     DPL,R1          ;R0,R1 = *vstk (start + used)
        MOV     DPH,R0
        MOVX    A,@DPTR
        MOV     R2,A            ;R2 = *stk - #OSStkStart + (used bzw. #OSFrameSize)
        MOV     A,R1
        CLR     C
        SUBB    A,R2
        MOV     DPL,A
        MOV     A,R0
        SUBB    A,#0            ;R2   = *stk - #OSStkStart
        MOV     DPH,A           ;DPTR = *vstk	
        save_stack 
        MOV     DPTR,#(OSTCBCur);R0,R1 = *vstk
        INC     DPTR            ;over segment-type
        MOVX    A,@DPTR         ;ptr = OSTCBCur-StkPtr
        MOV     R2,A
        INC     DPTR
        MOVX    A,@DPTR
        MOV     R3,A
        MOV     DPL,R3
        MOV     DPH,R2
        INC     DPTR            ;over segment-type
        MOV     A,R0
        MOVX    @DPTR,A
        INC     DPTR
        MOV     A,R1
        MOVX    @DPTR,A
        LJMP    OSStartHighRdy  ;to rest of activate task
ENDP

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
OSIntCtxSw      PROC
        CLR     EA              ;Do not interrupt during this sequence
        CLR     C
        MOV     A,SP
        SUBB    A,#4            ;correct 4 bytes on stack for two calls (four bytes)
        MOV     SP,A            ;Save the stackpointer

        POP     DPL             ;Optimised
        POP     DPH             ;Start testoring the context
        PUSH    DPH             ;
        PUSH    DPL             ;

        LJMP    ctxSwI          ;to rest of context-switch
ENDP

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
OSTickISR       PROC
;-------
        CLR     TR0             ;stop timer
        MOV     TL0,#T0_RELOADL ;reload Timer
        MOV     TH0,#T0_RELOADH ;reload Timer
        SETB    TR0             ;restart timer
;-------
        save_context
        LCALL   OSIntEnter
        LCALL   OSSystemTick
        LCALL   OSIntExit       ;perform a context-switch if nesesary
        restore_context
ENDP

END



