/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                   OS_AT32F4xx_c.c                        *
*                CORTEX-M4 Specific Code                   *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

#if (defined ( __FPU_USED ) && ( __FPU_USED == 1U ))
 #define USE_FPU 1
#else
 #define USE_FPU 0
#endif

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;
#if USE_FPU
    U08          i;
#endif

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
#if USE_FPU   
    *--stk = (OS_STK_TYPE)0xCCL; *--stk = (OS_STK_TYPE)0xDDL;
    for(i = 0; i < 16; i++)      *--stk = (OS_STK_TYPE)0xAAAAAAAAL;   /* S0-S15                   */
#endif
    *--stk   = (OS_STK_TYPE)0x01000000L;                  /* xPSR                                 */
    *--stk   = (OS_STK_TYPE)(task);                       /* task start address                   */
    *--stk   = (OS_STK_TYPE)0xFFFFFFF9L;                  /* R14 (LR)                             */
    *--stk   = (OS_STK_TYPE)0x12121212L;                  /* R12                                  */
    *--stk   = (OS_STK_TYPE)0x03030303L;                  /* R3                                   */
    *--stk   = (OS_STK_TYPE)0x02020202L;                  /* R2                                   */
    *--stk   = (OS_STK_TYPE)0x01010101L;                  /* R1                                   */
    *--stk   = (OS_STK_TYPE)Tdata;                        /* R0 : argument                        */
    *--stk   = (OS_STK_TYPE)0x11111111L;                  /* R11                                  */
    *--stk   = (OS_STK_TYPE)0x10101010L;                  /* R10                                  */
    *--stk   = (OS_STK_TYPE)0x09090909L;                  /* R9                                   */
    *--stk   = (OS_STK_TYPE)0x08080808L;                  /* R8                                   */
    *--stk   = (OS_STK_TYPE)0x07070707L;                  /* R7                                   */
    *--stk   = (OS_STK_TYPE)0x06060606L;                  /* R6                                   */
    *--stk   = (OS_STK_TYPE)0x05050505L;                  /* R5                                   */
    *--stk   = (OS_STK_TYPE)0x04040404L;                  /* R4                                   */
#if USE_FPU    
    for (i = 0; i < 16; i++)  *--stk = (OS_STK_TYPE)0xBBBBBBBBL;  /* S16-S31                      */
#endif
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
