/*************************************************************
*
* Description:
*	ATMega32 - Example program using ICC-AVR
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define	TASK_STK_SIZE   128

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];

U08 OS_SystemTicksCnt;

void  Task1(void *Tdata);
void  Task2(void *Tdata);

void main(void)
{
    OS_Init();

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 1);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 2);

    OS_Start();
}

void OSTickISR2(void)
{
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{
    OS_SystemTicksCnt = 0;
    OS_ENTER_CRITICAL();              // we use Timer0 normal, prescaler 64
    TCCR0   = 0x00;                   // stop
    TCNT0   = 0x00;                   // set start-count
    OCR0    = 0x00;                   // f = fosz /(2 * (1+ OCR0) * N ) mit N als Prescalfaktor 
    TCCR0   = 0x03;                   // start timer
    TIMSK   = 0x01;
    OS_EXIT_CRITICAL();
}

void Task1(void *Tdata)
{
    U08 LEDs = 0xFE;                                       // first ON bit-0

    Tdata = Tdata;
    AppTickInit();
    while(1) {
        PORTB = LEDs;
        OS_TimeDly(5);                      // test timer-wakeup
        LEDs ^= 0x01;                       // Toggle LED
    }
}

void Task2(void *Tdata)
{
    U08 LEDs = 0xFE;                                       // first ON bit-0

    Tdata = Tdata;
    while(1) {
        PORTB = LEDs;
        OS_TimeDly(5);                      // test timer-wakeup
        LEDs ^= 0x01;                       // Toggle LED
    }
}
