//***************************************************************
//*                       pC/OS 1.90b                           *
//*                      a FREE - RTOS                          *
//*                                                             *
//*                 based on the C/OS 1.00                     *
//*                   by  Jean J. Labrosse                      *
//*                                                             *
//*    changed by Frank Goetze  -  www.embedded-os.de           *
//***************************************************************
//*                      OS_Apollo3_a.s                         *
//*             Apollo3 Cortex-M4 Specific Code                 *
//*                        using IAR                            *
//***************************************************************

OS_NVIC_SYS_PRIx          EQU     0xE000ED20
OS_NVIC_SYS_PRIx_PENDSV   EQU     0x00FF0000
OS_NVIC_INT_CTRL          EQU     0xE000ED04
OS_NVIC_INT_CTRL_PENDSV   EQU     0x10000000

        EXTERN  OSTCBCur
        EXTERN  OSTCBHighRdy

    SECTION .text:CODE:REORDER(2)

//***************************************************************
//                     START MULTITASKING
//                  void OSStartHighRdy(void)
//***************************************************************
    PUBLIC OSStartHighRdy
OSStartHighRdy
        LDR     R0, =OS_NVIC_SYS_PRIx      ; set PendSV-exception priority lowest
        LDR     R1, [R0]
        ORR     R1, R1, #OS_NVIC_SYS_PRIx_PENDSV
        STR     R1, [R0]
        MOV     R0, #0                     ; set PSP for initial PendSV-exception
        MSR     PSP, R0
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        CPSIE   I                          ; enable interrupts
        BX      LR                         ; (for debugging)

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From task level)
//                      void OSCtxSw(void)
//***************************************************************
    PUBLIC OSCtxSw
OSCtxSw
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        BX      LR

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From an ISR)
//                    void OSIntCtxSw(void)
//***************************************************************
    PUBLIC OSIntCtxSw
OSIntCtxSw
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception (executed after leaving interrupt)
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        BX      LR

//***************************************************************
//                    HANDLE CONTEXT-SWITCH
//                   void OSPendSV_ISR(void)
//***************************************************************
//   ==> this must be hanged into INT-table 
//       offset-14 (PendSV handler)
//***************************************************************
    PUBLIC OSPendSV_ISR
OSPendSV_ISR
        CPSID   I
        MRS     R0, PSP                    ; check initial PendSV-exception
        CBZ     R0, OSPendISR_nosave       ; skip register-save the first time
        STMFD   R0!, {R4-R11, R14}         ; save r4-r11, r14 (LR) on process stack
    
    #if defined (__ARMVFP__)
        ORR     LR, LR, #0x10              ; set exception return not uses floating-point
        VSTMDB  R0!, {D8-D15}              ; save FPU regs d8-d15(s16-s31) on process 
    #endif

        LDR     R1, =OSTCBCur              ; save stack ptr in TCB
        LDR     R1, [R1]
        STR     R0, [R1]

OSPendISR_nosave
        LDR     R2, =OSTCBHighRdy          ; get TCB of highest priority task
        LDR     R1, =OSTCBCur              ; save as current 
        LDR     R2, [R2]
        STR     R2, [R1]
        LDR     R0, [R2]                   ; get new stack-ptr
        
    #if defined (__ARMVFP__)					
        VLDMIA  R0!, {D8-D15}              ; restore FPU regs d8-d15(s16-s31) from new process stack
        BIC.W   LR, LR, #0x10              ; set exception return uses floating-point 
    #endif

        LDMFD   R0!, {R4-R11, R14}         ; restore r4-11, r14 (LR) from new process stack
        MSR     PSP, R0                    ; load PSP with new stack-ptr
        ORR     LR, LR, #0x04              ; ensure exception return uses process stack
        CPSIE   I
        BX      LR

//***************************************************************
//                       HANDLE TICK ISR
//                     void OSTickISR(void)
//***************************************************************
//                       not needed here
//***************************************************************

//***************************************************************
//                    SPECIAL IRQ-ISR (ALL)
//                     void OSirqISR(void)
//***************************************************************
//                       not needed here
//***************************************************************


//***************************************************************
//                    SPECIAL INT-SAVE CALLS
//                  OS_STK_TYPE OSEnterCrit(void)
//                  void OSExitCrit(OS_STK_TYPE)
//***************************************************************
    PUBLIC OSEnterCrit
OSEnterCrit
        MRS     R0, PRIMASK
        CPSID   I
        BX      LR                          ; return R0

    PUBLIC OSExitCrit
OSExitCrit
        MSR     PRIMASK, R0
        BX      LR                          ; return


//***************************************************************
      END


