/*************************************************************
*
* Description:
*   CH32V103_ - Example program using MounRiver Studio for RISC-V
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"


#define LED_RCC   RCC_APB2Periph_GPIOB
#define LED_GPIO  GPIOB
#define LED_PIN   GPIO_Pin_15

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         512
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif
OS_P        Pipe1;
OS_SMQ      MQueue1;
OS_STK_TYPE MQueue_d[16];

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

static inline U32 SysTick_Config(U32 ticks)
{
    if ((ticks - 1) > 0xFFFFFFFF)
    {
        return 1;
    }
    ticks -= 1;

    SysTick->CTLR = 0;

    SysTick->CNTL0 = 0;
    SysTick->CNTL1 = 0;
    SysTick->CNTL2 = 0;
    SysTick->CNTL3 = 0;

    SysTick->CNTH0 = 0;
    SysTick->CNTH1 = 0;
    SysTick->CNTH2 = 0;
    SysTick->CNTH3 = 0;

    SysTick->CMPLR0 = (ticks >>  0) & 0xff;
    SysTick->CMPLR1 = (ticks >>  8) & 0xff;
    SysTick->CMPLR2 = (ticks >> 16) & 0xff;
    SysTick->CMPLR3 = (ticks >> 24) & 0xff;

    SysTick->CMPHR0 = 0;
    SysTick->CMPHR1 = 0;
    SysTick->CMPHR2 = 0;
    SysTick->CMPHR3 = 0;

    SysTick->CTLR = 1;

    NVIC_SetPriority(SysTicK_IRQn,0xff);
    NVIC_EnableIRQ(SysTicK_IRQn);

    return 0;
}

void SysTick_init(void)
{
    SystemCoreClockUpdate();
    if (SysTick_Config(HSE_VALUE / OS_TIMER_TICKS_PER_SEC)) {
        while (1) _NOP();
    }
}

void SysTick_update(void)
{
    NVIC_ClearPendingIRQ(SysTicK_IRQn);
    SysTick->CTLR = 0;
    SysTick->CNTL0 = 0;
    SysTick->CNTL1 = 0;
    SysTick->CNTL2 = 0;
    SysTick->CNTL3 = 0;
    SysTick->CTLR = 1;
}

void LED_init(void)
{
    GPIO_InitTypeDef    GPIO_InitStructure = {0};

    RCC_APB2PeriphClockCmd(LED_RCC, ENABLE);
    GPIO_InitStructure.GPIO_Pin = LED_PIN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(LED_GPIO, &GPIO_InitStructure);

    GPIO_Write(LED_GPIO, LED_PIN);
    GPIO_Write(LED_GPIO, (U16)~GPIO_ReadOutputData(LED_GPIO));   // switch ON (control)
}

void LED_toggle(void)
{
    GPIO_WriteBit(LED_GPIO, LED_PIN, ~GPIO_ReadOutputData(LED_GPIO) & LED_PIN);
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    NVIC_SetPriority(Software_IRQn,0xff);
    NVIC_EnableIRQ(Software_IRQn);
    // ---------------------
    
    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif
    
    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical RISC-V pC/OS ISR (called from SysTick_Handler)
{
    OS_IntEnter();

    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
    OS_IntExit();
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i, tmq[10];
    void *t = (void *)tmq;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);

    returnOk=OS_PipeInit(&Pipe1, NULL, 0, 0);
    returnOk=OS_PipePost(&Pipe1, (U08 OS_HUGE *)"0123456789", 10, OS_SUSPEND);
    returnOk=OS_PipePost(&Pipe1, (U08 OS_HUGE *)"abcdefghijklmnopqrst", 20, OS_SUSPEND);
    returnOk=OS_PipeFrontPost(&Pipe1, (U08 OS_HUGE *)".,-+*", 5, OS_SUSPEND);

    returnOk=OS_SMQueueInit(&MQueue1, MQueue_d, 16);
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);
    returnOk=OS_SMQueuePost(&MQueue1, (void **)&ptr, OS_SUSPEND);
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 20);
    returnOk=OS_SMQueueFrontPost(&MQueue1, (void **)&ptr, OS_SUSPEND);
    returnOk=OS_SMQueuePost(&MQueue1, &t, OS_SUSPEND);

    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk, msg[30];
    U32 i;
    U16 l, *mq;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {

            returnOk=OS_PipePend(&Pipe1, msg, &l, OS_NO_SUSP);

            returnOk=OS_SMQueuePend(&MQueue1, (void **)&mq, OS_NO_SUSP);
            if ((mq != NULL) && (!OS_MemVerifyPtr(mq)))
                returnOk=OS_MemFree((void **)&mq);

            i=0;
        	while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;
 
    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif    
    
    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
