/***************************************************************
 *                       pC/OS 1.95c                           *
 *                      a FREE - RTOS                          *
 *                                                             *
 *                 based on the µC/OS 1.00                     *
 *                   by  Jean J. Labrosse                      *
 *                                                             *
 *    changed by Frank Goetze  -  www.embedded-os.de           *
 ***************************************************************
 *                     OS_CH32V20x_a.s                         *
 *               CH32V20x RISC-V Specific Code                 *
 *                       using GCC                             *
 ***************************************************************/

    .equ    Software_IRQn , 14
    .equ    SW_IRQn_BIT   , 1 << Software_IRQn
    .equ    PFIC_SW_IPSR  , 0xE000E000 + 0x200

    .equ    mscratch      , 0x340

    .extern  OSTCBCur
    .extern  OSTCBHighRdy
    .extern  OSTickISR

    .text
    .align 4

 /***************************************************************
 *                         marcos
 **************************************************************/
    .macro store_context            /* save all register x31..x0  (expect x0,2,3) */
        addi    sp,  sp, -30*4
        sw      x5,   2*4(sp)

        csrr    t0,  mstatus
        sw      t0,  29*4(sp)

        /*
         interrupts could be still enabled in ISRs ... (for neesting see INTSYSCR)
         so disable they as fast as possible
         up to the point neesting can/could be allowed after context-save
        */
        csrc    mstatus, 8

        sw      x1,   0*4(sp)
        sw      x4,   1*4(sp)
        sw      x6,   3*4(sp)
        sw      x7,   4*4(sp)
        sw      x8,   5*4(sp)
        sw      x9,   6*4(sp)
        sw      x10,  7*4(sp)
        sw      x11,  8*4(sp)
        sw      x12,  9*4(sp)
        sw      x13, 10*4(sp)
        sw      x14, 11*4(sp)
        sw      x15, 12*4(sp)
        sw      x16, 13*4(sp)
        sw      x17, 14*4(sp)
        sw      x18, 15*4(sp)
        sw      x19, 16*4(sp)
        sw      x20, 17*4(sp)
        sw      x21, 18*4(sp)
        sw      x22, 19*4(sp)
        sw      x23, 20*4(sp)
        sw      x24, 21*4(sp)
        sw      x25, 22*4(sp)
        sw      x26, 23*4(sp)
        sw      x27, 24*4(sp)
        sw      x28, 25*4(sp)
        sw      x29, 26*4(sp)
        sw      x30, 27*4(sp)
        sw      x31, 28*4(sp)

        csrr    t0,  mepc
        sw      t0,  30*4(sp)

        la      t0,  OSTCBCur
        lw      t0,  0(t0)
        sw      sp,  0(t0)
    .endm

    .macro restore_context            /* restore all register x0..x31 (expect x0,2,3) */
        lw      x1,   0*4(sp)
        lw      x4,   1*4(sp)

        lw      x6,   3*4(sp)
        lw      x7,   4*4(sp)
        lw      x8,   5*4(sp)
        lw      x9,   6*4(sp)
        lw      x10,  7*4(sp)
        lw      x11,  8*4(sp)
        lw      x12,  9*4(sp)
        lw      x13, 10*4(sp)
        lw      x14, 11*4(sp)
        lw      x15, 12*4(sp)
        lw      x16, 13*4(sp)
        lw      x17, 14*4(sp)
        lw      x18, 15*4(sp)
        lw      x19, 16*4(sp)
        lw      x20, 17*4(sp)
        lw      x21, 18*4(sp)
        lw      x22, 19*4(sp)
        lw      x23, 20*4(sp)
        lw      x24, 21*4(sp)
        lw      x25, 22*4(sp)
        lw      x26, 23*4(sp)
        lw      x27, 24*4(sp)
        lw      x28, 25*4(sp)
        lw      x29, 26*4(sp)
        lw      x30, 27*4(sp)
        lw      x31, 28*4(sp)

        lw      t0,  29*4(sp)
        csrw    mstatus, t0

        lw      t0,  30*4(sp)
        csrw    mepc, t0

        lw      x5,   2*4(sp)
        addi    sp, sp, 30*4
     .endm

/***************************************************************
 *                     START MULTITASKING
 *                  void OSStartHighRdy(void)
 ***************************************************************/
    .global OSStartHighRdy
OSStartHighRdy:
        la      t0, _eusrstack
        addi    t0, t0, -256
        csrw    mscratch, t0
        la      t0, OSTCBHighRdy
        lw      t0, 0(t0)
        lw      sp, 0(t0)
        restore_context
        mret

/***************************************************************
 *         INITIATE A CONTEXT SWITCH (From task level)
 *                    void OSCtxSw(void)
 ***************************************************************/
    .global OSCtxSw
OSCtxSw:
        li      t0, PFIC_SW_IPSR
        li      t1, SW_IRQn_BIT
        sw      t1, (t0)
        ret

/***************************************************************
 *            PERFORM A CONTEXT SWITCH (From an ISR)
 *                   void OSIntCtxSw(void)
 ***************************************************************/
    .global OSIntCtxSw
OSIntCtxSw:
        csrrw   sp, mscratch, sp
        la      t0, OSTCBHighRdy
        lw      t0, 0(t0)
        la      t1, OSTCBCur
        sw      t0, 0(t1)
        lw      sp, 0(t0)
        restore_context
        mret

/***************************************************************
 *                   HANDLE CONTEXT-SWITCH
 *           PERFORM A CONTEXT SWITCH (From task level)
 *                        IRQ-HANDLER
 ***************************************************************/
    .global OSCtxSw_ISR
OSCtxSw_ISR:
        store_context
        csrrw   sp, mscratch, sp
        li      t0, PFIC_SW_IPSR
        sw      zero, (t0)
        j       OSIntCtxSw

/***************************************************************
 *                       HANDLE TICK ISR
 *                        IRQ-HANDLER
 *                     void OSTickISR(void)
 ***************************************************************/
    .global OSTick_ISR
OSTick_ISR:
        store_context
        csrrw   sp, mscratch, sp
        jal OSTickISR
        csrrw   sp, mscratch, sp
        restore_context
        mret

/***************************************************************
 *                  SPECIAL INT-SAVE CALLS
 *                OS_STK_TYPE OSEnterCrit(void)
 *                void OSExitCrit(OS_STK_TYPE)
 ***************************************************************/
    .global OSEnterCrit
OSEnterCrit:
        csrrc   a0, mstatus, 8
        ret

    .global OSExitCrit
OSExitCrit:
        csrw    mstatus, a0
        ret


/***************************************************************/


