/***************************************************************
 *                       pC/OS 1.91c                           *
 *                      a FREE - RTOS                          *
 *                                                             *
 *                 based on the C/OS 1.00                     *
 *                   by  Jean J. Labrosse                      *
 *                                                             *
 *    changed by Frank Goetze  -  www.embedded-os.de           *
 ***************************************************************
 *                    OS_GD32VF103C_a.s                        *
 *              GD32VF103C RISC-V Specific Code                *
 *                       using GCC                             *
 ***************************************************************/
#define ECLIC_SW_MSIP 0xd1000FFC    /* TIMER_MSIP */

#define CSR_MSTATUS        0x300
#define CSR_MEPC           0x341
#define CSR_MSUBM          0x7c4
#define CSR_JALMNXTI       0x7ED

    .extern  OSTCBCur
    .extern  OSTCBHighRdy
    .extern  OSTickISR

    .text
    .align 4

 /***************************************************************
 *                         marcos
 **************************************************************/
    .macro store_context            // save all register x31..x0  (expect x2)
        addi    sp, sp, -35*4

        sw      x0,   0*4(sp)
        sw      x1,   1*4(sp)
        sw      x3,   3*4(sp)
        sw      x4,   4*4(sp)
        sw      x5,   5*4(sp)
        sw      x6,   6*4(sp)
        sw      x7,   7*4(sp)
        sw      x8,   8*4(sp)
        sw      x9,   9*4(sp)
        sw      x10, 10*4(sp)
        sw      x11, 11*4(sp)
        sw      x12, 12*4(sp)
        sw      x13, 13*4(sp)
        sw      x14, 14*4(sp)
        sw      x15, 15*4(sp)

        sw      x16, 16*4(sp)
        sw      x17, 17*4(sp)
        sw      x18, 18*4(sp)
        sw      x19, 19*4(sp)
        sw      x20, 20*4(sp)
        sw      x21, 21*4(sp)
        sw      x22, 22*4(sp)
        sw      x23, 23*4(sp)
        sw      x24, 24*4(sp)
        sw      x25, 25*4(sp)
        sw      x26, 26*4(sp)
        sw      x27, 27*4(sp)
        sw      x28, 28*4(sp)
        sw      x29, 29*4(sp)
        sw      x30, 30*4(sp)
        sw      x31, 31*4(sp)

        csrr    x10, mstatus
        sw      x10, 32*4(sp)
        csrr    x10, mepc
        sw      x10, 33*4(sp)
        csrr    x10, CSR_MSUBM
        sw      x10, 34*4(sp)
/*
  csrr x5, CSR_MEPC
  STORE x5,  16*REGBYTES(sp)
  csrr x5, CSR_MSTATUS
  STORE x5,  17*REGBYTES(sp)
  csrr x5, CSR_MSUBM
  STORE x5,  18*REGBYTES(sp)
*/

    .endm

    .macro restore_context            // restore all register x0..x31 (expect x2)
        lw      x0,   0*4(sp)
        lw      x1,   1*4(sp)
        lw      x3,   3*4(sp)
        lw      x4,   4*4(sp)
        lw      x5,   5*4(sp)
        lw      x6,   6*4(sp)
        lw      x7,   7*4(sp)
        lw      x8,   8*4(sp)
        lw      x9,   9*4(sp)
        lw      x11, 11*4(sp)
        lw      x12, 12*4(sp)
        lw      x13, 13*4(sp)
        lw      x14, 14*4(sp)
        lw      x15, 15*4(sp)

        lw      x16, 16*4(sp)
        lw      x17, 17*4(sp)
        lw      x18, 18*4(sp)
        lw      x19, 19*4(sp)
        lw      x20, 20*4(sp)
        lw      x21, 21*4(sp)
        lw      x22, 22*4(sp)
        lw      x23, 23*4(sp)
        lw      x24, 24*4(sp)
        lw      x25, 25*4(sp)
        lw      x26, 26*4(sp)
        lw      x27, 27*4(sp)
        lw      x28, 28*4(sp)
        lw      x29, 29*4(sp)
        lw      x30, 30*4(sp)
        lw      x31, 31*4(sp)

        lw      x10, 32*4(sp)
        csrw    mstatus, x10
        lw      x10, 33*4(sp)
        csrw    mepc, x10
        lw      x10, 34*4(sp)
        csrw    CSR_MSUBM, x10
/*
  LOAD x5,  16*REGBYTES(sp)
  csrw CSR_MEPC, x5
  LOAD x5,  17*REGBYTES(sp)
  csrw CSR_MSTATUS, x5
  LOAD x5,  18*REGBYTES(sp)
  csrw CSR_MSUBM, x5
*/

        lw      x10, 10*4(sp)
        addi    sp, sp, 35*4
     .endm

/***************************************************************
 *                     START MULTITASKING
 *                  void OSStartHighRdy(void)
 ***************************************************************/
    .global OSStartHighRdy
OSStartHighRdy:
        la      t0, OSTCBHighRdy
        lw      t0, 0(t0)
        lw      sp, 0(t0)
        restore_context
        mret

/***************************************************************
 *         INITIATE A CONTEXT SWITCH (From task level)
 *                    void OSCtxSw(void)
 ***************************************************************/
    .global OSCtxSw
OSCtxSw:
        li      t0, ECLIC_SW_MSIP
        li      t1, 1
        sw      t1, (t0)
        ret

/***************************************************************
 *            PERFORM A CONTEXT SWITCH (From an ISR)
 *                   void OSIntCtxSw(void)
 ***************************************************************/
    .global OSIntCtxSw
OSIntCtxSw:
        la      t0, OSTCBHighRdy
        lw      t0, 0(t0)
        la      t1, OSTCBCur
        sw      t0, 0(t1)
        lw      sp, 0(t0)
        restore_context
        mret

/***************************************************************
 *                   HANDLE CONTEXT-SWITCH
 *           PERFORM A CONTEXT SWITCH (From task level)
 *                        IRQ-HANDLER
 ***************************************************************/
    .global OSCtxSw_ISR
OSCtxSw_ISR:
        li      t0, ECLIC_SW_MSIP
        sw      zero, (t0)

        la      t0, OSTCBCur
        lw      t0, 0(t0)
        sw      sp, 0(t0)
        j       OSIntCtxSw

/***************************************************************
 *                      HANDLE CORE-TIMER
 *                 PERFORM CORE-TIMER HANDLING
 *                        IRQ-HANDLER
 ***************************************************************
 *                      not needed here
 ***************************************************************/

/***************************************************************
 *             SPECIAL EXCEPTIONs / INTERRUPTs VECTOR
 *                all interrupts entry Handler
 *                   Jumps to handle_intr
 ***************************************************************/
    .section      .text.entry
    .align 4
    .global OSintrEntry
OSintrEntry:                          // referenced in "start.s" CSR_MTVT2
        store_context

        csrr    a0, mcause
        li      t0, 0xb8000003
        beq     a0, t0, OSCtxSw_ISR

        la      t0, OSIntNesting
        lw      t1, 0(t0)
        add     t2, t1, 1
        sw      t2, 0(t0)
        bnez    t1, 1f

        la      t0, OSTCBCur
        lw      t0, 0(t0)
        sw      sp, 0(t0)
#if OWN_TRAP_STACK
        la      sp, _sp
#endif
1:
  //------This special CSR read/write operation, which is actually Claim the CLIC to find its pending highest
  // ID, if the ID is not 0, then automatically enable the mstatus.MIE, and jump to its vector-entry-label, and
  // update the link register
        csrrw   ra, CSR_JALMNXTI, ra

//        call    handle_intr

        call    OSIntExit
#if OWN_TRAP_STACK
        la      t0, OSIntNesting
        lw      t0, 0(t0)
        bnez    t0, 1f

        la      t0, OSTCBCur
        lw      t0, 0(t0)
        lw      sp, 0(t0)
1:
#endif
        restore_context
        mret

/***************************************************************
 *                  SPECIAL INT-SAVE CALLS
 *                OS_STK_TYPE OSEnterCrit(void)
 *                void OSExitCrit(OS_STK_TYPE)
 ***************************************************************/
    .global OSEnterCrit
OSEnterCrit:
        csrrc   a0, mstatus, 8
        ret

    .global OSExitCrit
OSExitCrit:
        csrw    mstatus, a0
        ret


/***************************************************************/


