//***************************************************************
//*                       pC/OS 1.70b                           *
//*                      a FREE - RTOS                          *
//*                                                             *
//*                 based on the C/OS 1.00                     *
//*                   by  Jean J. Labrosse                      *
//*                                                             *
//*    changed by Frank Goetze  -  www.embedded-os.de           *
//***************************************************************
//*                     OS_LM3S811_a.s79                        *
//*             LM3S811 Cortex-M3 Specific Code                 *
//*                        using IAR                            *
//***************************************************************
#include "header/asmdefs.h"                ; EW-ARM / GCC / RV-MDK  asm-defines

#define OS_NVIC_SYS_PRIx          0xE000ED20
#define OS_NVIC_SYS_PRIx_PENDSV   0x00FF0000
#define OS_NVIC_INT_CTRL          0xE000ED04
#define OS_NVIC_INT_CTRL_PENDSV   0x10000000

        __IMPORT__  OSTCBCur
        __IMPORT__  OSTCBHighRdy

    __TEXT__

//***************************************************************
//                     START MULTITASKING
//                  void OSStartHighRdy(void)
//***************************************************************
    __EXPORT__ OSStartHighRdy
    __THUMB_LABEL__
OSStartHighRdy __LABEL__
        LDR     R0, =OS_NVIC_SYS_PRIx      ; set PendSV-exception priority
        LDR     R1, [R0]
        ORR     R1, R1, #OS_NVIC_SYS_PRIx_PENDSV
        STR     R1, [R0]
        MOV     R0, #0                     ; set PSP for initial PendSV-exception
        MSR     PSP, R0
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        CPSIE   I                          ; enable interrupts
        BX      LR                         ; (for debugging)

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From task level)
//                      void OSCtxSw(void)
//***************************************************************
    __EXPORT__ OSCtxSw
    __THUMB_LABEL__
OSCtxSw __LABEL__
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        BX      LR

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From an ISR)
//                    void OSIntCtxSw(void)
//***************************************************************
    __EXPORT__ OSIntCtxSw
    __THUMB_LABEL__
OSIntCtxSw __LABEL__
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        BX      LR

//***************************************************************
//                    HANDLE CONTEXT-SWITCH
//                   void OSPendSV_ISR(void)
//***************************************************************
//   ==> this must be hanged into INT-table 
//       offset-14 (PendSV handler)
//***************************************************************
    __EXPORT__ OSPendSV_ISR
    __THUMB_LABEL__
OSPendSV_ISR __LABEL__
        MRS     R0, PSP                    ; check initial PendSV-exception
        CBZ     R0, OSPendISR_nosave       ; skip register-save the first time
        SUB     R0, R0, #0x20
        STM     R0, {R4-R11}               ; save r4-r11
        LDR     R1, =OSTCBCur              ; save stack ptr in TCB
        LDR     R1, [R1]
        STR     R0, [R1]
OSPendISR_nosave
        LDR     R2, =OSTCBHighRdy          ; get TCB of highest priority task
        LDR     R1, =OSTCBCur              ; save as current 
        LDR     R2, [R2]
        STR     R2, [R1]
        LDR     R0, [R2]                   ; get new stack-ptr
        LDM     R0, {R4-R11}               ; restore r4-r11
        ADD     R0, R0, #0x20
        MSR     PSP, R0                    ; load PSP with new stack-ptr
        ORR     LR, LR, #0x04              ; ensure exception return uses process stack
        BX      LR

//***************************************************************
//                       HANDLE TICK ISR
//                     void OSTickISR(void)
//***************************************************************
//                       not needed here
//***************************************************************

//***************************************************************
//                    SPECIAL IRQ-ISR (ALL)
//                     void OSirqISR(void)
//***************************************************************
//                       not needed here
//***************************************************************


//***************************************************************
//                    SPECIAL INT-SAVE CALLS
//                  OS_STK_TYPE OSEnterCrit(void)
//                  void OSExitCrit(OS_STK_TYPE)
//***************************************************************
    __EXPORT__ OSEnterCrit
    __THUMB_LABEL__
OSEnterCrit __LABEL__
        MRS     R0, PRIMASK
        CPSID   I
        BX      LR                          ; return R0

    __EXPORT__ OSExitCrit
    __THUMB_LABEL__
OSExitCrit __LABEL__
        MSR     PRIMASK, R0
        BX      LR                          ; return


//***************************************************************
    __END__


