/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the µC/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                    OS_IPC_LPC4337.c                      *
*               Dual-Core IPC Specific Code                *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

#define OS_ICC_SIZE  (48 + sizeof(U16))   // bytes per message + sizeof(U16)
#define OS_ICC_DEEP   16                  // max messages in ICC AMP-Pipe

//volatile struct shared_data * const shared_ptr = (struct shared_data *)0x30040000;

/***********************************************************
* creating both pipes channels and management structs in   *
* shared memory for both cores                             *
*                                                          *
* using sections                                           *
*   static OS_ICC ipc_0                               __attribute__((section("shared_memory"))) = {0};
*   static OS_ICC ipc_1                               __attribute__((section("shared_memory"))) = {0};
*   static U08 icc_0_data[(OS_ICC_SIZE * OS_IPC_DEEP] __attribute__((section("shared_memory"))) = {0};
*   static U08 icc_1_data[(OS_ICC_SIZE * OS_IPC_DEEP] __attribute__((section("shared_memory"))) = {0};
*                                                          *
* using manaul adress settings                             *
* (offset calculated with padding space between)           *
* like LPC4337 using manual shared memory region "0x10080000"
*   static OS_IPC  *IccRxS = (OS_ICC *)0x10080000;    // icc_0_shared;
*   static OS_IPC  *IccTxS = (OS_ICC *)0x10080080;    // icc_1_shared;
*   static U08 *IccTxDataS =    (U08 *)0x10080100;    // icc_0_shared_data;
*   static U08 *IccTxDataS =    (U08 *)0x10080500;    // icc_1_shared_data;
*                                                          *
***********************************************************/

#ifdef CORE_M4
 static OS_ICC  *IccRxS = (volatile OS_ICC *)0x10080000;
 static OS_ICC  *IccTxS = (volatile OS_ICC *)0x10080080;
 static U08 *IccRxDataS =    (U08 *)0x10080100;
 static U08 *IccTxDataS =    (U08 *)0x10080500;
 #define ICC_IRQHandler  M0APP_IRQHandler
 #define ICC_IRQn        M0APP_IRQn
 #define ICC_EventClr    M0APPTXEVENT
#endif 
#ifdef CORE_M0
 static OS_ICC  *IccRxS = (volatile OS_ICC *)0x10080080;
 static OS_ICC  *IccTxS = (volatile OS_ICC *)0x10080000;
 static U08 *IccRxDataS =    (U08 *)0x10080500;
 static U08 *IccTxDataS =    (U08 *)0x10080100;
 #define ICC_IRQHandler  M0_M4CORE_IRQHandler
 #define ICC_IRQn        M0_M4CORE_IRQn
 #define ICC_EventClr    M4TXEVENT
#endif
 #define ICC_Event()     { __DSB(); __SEV(); }

void ICC_IRQEnable(void)
{
    NVIC_ClearPendingIRQ(ICC_IRQn);
    NVIC_EnableIRQ(ICC_IRQn);
}

void ICC_IRQHandler(void)
{
    OS_IntEnter();
    LPC_CREG->ICC_EventClr = 0;
    NVIC_ClearPendingIRQ(ICC_IRQn);
    OS_IccEventHandler();
    OS_IntExit();
}

