;/***************************************************************
;/*                       pC/OS 1.91c                           *
;/*                      a FREE - RTOS                          *
;/*                                                             *
;/*                 based on the C/OS 1.00                     *
;/*                   by  Jean J. Labrosse                      *
;/*                                                             *
;/*    changed by Frank Goetze  -  www.embedded-os.de           *
;/***************************************************************
;/*                     OS_LPC55S69_a.s                         *
;/*       ARMv8 mainline Cortex-M33 Specific Code (with TZ)     *
;/*                       using Keil                            *
;/***************************************************************

OS_NVIC_SHPR3             EQU     0xE000ED20
OS_NVIC_SHPR3_PENDSV      EQU     0x00FF0000
OS_NVIC_ICSR              EQU     0xE000ED04
OS_NVIC_ICSR_PENDSV       EQU     0x10000000

        IMPORT  OSTCBCur
        IMPORT  OSTCBHighRdy

    PRESERVE8
    THUMB
    AREA    |.text|, CODE, READONLY

;/***************************************************************
;/                     START MULTITASKING
;/                  void OSStartHighRdy(void)
;/***************************************************************
    EXPORT OSStartHighRdy
    THUMB
OSStartHighRdy
        LDR     R0, =OS_NVIC_SHPR3         ; set PendSV-exception priority lowest
        LDR     R1, [R0]
        ORR     R1, R1, #OS_NVIC_SHPR3_PENDSV
        STR     R1, [R0]
        MOV     R0, #0                     ; set PSP for initial PendSV-exception
        MSR     PSP, R0
        LDR     R0, =OS_NVIC_ICSR          ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_ICSR_PENDSV
        STR     R1, [R0]
        CPSIE   I                          ; enable interrupts
        BX      LR                         ; (for debugging)

;/***************************************************************
;/           PERFORM A CONTEXT SWITCH (From task level)
;/                      void OSCtxSw(void)
;/***************************************************************
    EXPORT OSCtxSw
    THUMB
OSCtxSw
        LDR     R0, =OS_NVIC_ICSR          ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_ICSR_PENDSV
        STR     R1, [R0]
        BX      LR

;/***************************************************************
;/           PERFORM A CONTEXT SWITCH (From an ISR)
;/                    void OSIntCtxSw(void)
;/***************************************************************
    EXPORT OSIntCtxSw
    THUMB
OSIntCtxSw
        LDR     R0, =OS_NVIC_ICSR          ; trigger PendSV-exception (executed after leaving interrupt) 
        LDR     R1, =OS_NVIC_ICSR_PENDSV
        STR     R1, [R0]
        BX      LR

;/***************************************************************
;/                    HANDLE CONTEXT-SWITCH
;/                   void OSPendSV_ISR(void)
;/***************************************************************
;/   ==> this must be hanged into INT-table 
;/       offset-14 (PendSV handler)
;/***************************************************************
    EXPORT OSPendSV_ISR
    THUMB
OSPendSV_ISR
        MRS     R0, PSP                    ; check initial PendSV-exception
        CBZ     R0, OSPendISR_nosave       ; skip register-save the first time
        MRS     R1, PSPLIM                 ; get PSPLIM
        MOV     R2, LR                     ; get LR
        STMFD   R0!, {R1,R2,R4-R11}        ; save PSPLIM, LR, r4-r11

    IF {FPU} != "SoftVFP"
        ORR     LR, LR, #0x10              ; set exception return not uses floating-point
        VSTMFD  R0!, {D8-D15}              ; save FPU regs d8-d15(s16-s31) on process 
    ENDIF

        LDR     R1, =OSTCBCur              ; save stack ptr in TCB
        LDR     R1, [R1]
        STR     R0, [R1]
OSPendISR_nosave
        LDR     R2, =OSTCBHighRdy          ; get TCB of highest priority task
        LDR     R1, =OSTCBCur              ; save as current 
        LDR     R2, [R2]
        STR     R2, [R1]
        LDR     R0, [R2]                   ; get new stack-ptr

    IF {FPU} != "SoftVFP"
        VLDMFD  R0!, {D8-D15}              ; restore FPU regs d8-d15(s16-s31) from new process stack
        BIC.W   LR, LR, #0x10              ; set exception return uses floating-point 
    ENDIF

        LDMFD   R0!, {R1,R2,R4-R11}        ; restore PSPLIM, LR, r4-11 from new process stack
        MOV     LR, R2                     ; load LR
        MSR     PSPLIM, R1                 ; load PSPLIM with new ptr
        MSR     PSP, R0                    ; load PSP with new stack-ptr
        BX      LR

;/***************************************************************
;/                       HANDLE TICK ISR
;/                     void OSTickISR(void)
;/***************************************************************
;/                       not needed here
;/***************************************************************

;/***************************************************************
;/                    SPECIAL IRQ-ISR (ALL)
;/                     void OSirqISR(void)
;/***************************************************************
;/                       not needed here
;/***************************************************************


;/***************************************************************
;/                    SPECIAL INT-SAVE CALLS
;/                  OS_STK_TYPE OSEnterCrit(void)
;/                  void OSExitCrit(OS_STK_TYPE)
;/***************************************************************
    EXPORT OSEnterCrit
    THUMB
OSEnterCrit
        MRS     R0, PRIMASK
        CPSID   I
        BX      LR                          ; return R0

    EXPORT OSExitCrit
    THUMB
OSExitCrit
        MSR     PRIMASK, R0
        BX      LR                          ; return

;/***************************************************************
    ALIGN 4
    END


