/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                   OS_MK20DX256_c.c                       *
*                CORTEX-M4 Specific Code                   *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

#define USE_FPU   /* ARMCC */ (  (defined ( __CC_ARM ) && defined ( __TARGET_FPU_VFP )) \
                  /* IAR */   || (defined ( __ICCARM__ ) && defined ( __ARMVFP__ )) \
                  /* GNU */   || (defined ( __GNUC__ ) && defined ( __VFP_FP__ ) && !defined(__SOFTFP__)) )

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;
#if USE_FPU
    U08          i;
#endif

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
#if USE_FPU   
    *--stk = (U32)0xCCL;    *--stk = (U32)0xDDL;
    for(i = 0; i < 16; i++)  *--stk = (U32)0xAAAAAAAAL;   /* S0-S15                               */
#endif
    *--stk   = (U32)0x01000000L;                          /* xPSR                                 */
    *--stk   = (U32)(task);                               /* task start address                   */
    *--stk   = (U32)0xFFFFFFF9L;                          /* R14 (LR)                             */
    *--stk   = (U32)0x12121212L;                          /* R12                                  */
    *--stk   = (U32)0x03030303L;                          /* R3                                   */
    *--stk   = (U32)0x02020202L;                          /* R2                                   */
    *--stk   = (U32)0x01010101L;                          /* R1                                   */
    *--stk   = (U32)Tdata;                                /* R0 : argument                        */
    *--stk   = (U32)0x11111111L;                          /* R11                                  */
    *--stk   = (U32)0x10101010L;                          /* R10                                  */
    *--stk   = (U32)0x09090909L;                          /* R9                                   */
    *--stk   = (U32)0x08080808L;                          /* R8                                   */
    *--stk   = (U32)0x07070707L;                          /* R7                                   */
    *--stk   = (U32)0x06060606L;                          /* R6                                   */
    *--stk   = (U32)0x05050505L;                          /* R5                                   */
    *--stk   = (U32)0x04040404L;                          /* R4                                   */
#if USE_FPU    
    for (i = 0; i < 16; i++)  *--stk = (U32)0xBBBBBBBBL;  /* S16-S31                              */
#endif
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
