;****************************************************************
;*                       pC/OS 1.70b                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                       OS_MSP430_a.s                          *
;*                 TI MSP430 Specific Code                      *
;*                  using TI-CodeComposer                       *
;****************************************************************
    .cdecls C,LIST,"msp430x16x.h"  ; Include device header file

    .global OSTCBHighRdy
    .global OSTCBCur
    .global OSIntNesting
    .global OSTickISR
    .global OSIntExit

    .global OSStartHighRdy
    .global OSCtxSw
    .global OSIntCtxSw
    .global OSEnterCrit
    .global OSExitCrit
    
    .data
    .align 2 
OSISRStkPtr:
    .short 1

    .text    
;****************************************************************
;                             MACROS
;****************************************************************
PUSHALL .macro
        push     r4
        push     r5
        push     r6
        push     r7
        push     r8
        push     r9
        push     r10
        push     r11
        push     r12
        push     r13
        push     r14
        push     r15         
        .endm

POPALL  .macro 
        pop      r15
        pop      r14
        pop      r13
        pop      r12
        pop      r11
        pop      r10
        pop      r9
        pop      r8
        pop      r7
        pop      r6
        pop      r5
        pop      r4          
        .endm

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
    .align  
OSStartHighRdy:
        mov.w    SP, &OSISRStkPtr       ; save interrupt stack              
        mov.w    &OSTCBHighRdy, R13     ; get TCB of highest priority task
        mov.w     R13, &OSTCBCur        ; save as current
        mov.w    @R13, SP               ; restore stack-ptr 
        POPALL 
        reti                            ; emulate return from interrupt

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
    .align  
OSCtxSw:
        push      sr                    ; emulate interrupt by also saving the SR
        PUSHALL                         ; push all registers             
        mov.w     &OSTCBCur, R13        ; Save stack ptr in TCB
        mov.w     SP, 0(R13)
        mov.w     &OSTCBHighRdy, R13    ; get TCB of highest priority task
        mov.w     R13, &OSTCBCur        ; save as current
        mov.w     @R13, SP              ; restore new stack-ptr
        POPALL
        reti                            ; emulate return from interrupt

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
    .align  
OSIntCtxSw:
        mov.w     &OSTCBHighRdy, R13    ; get TCB of highest priority task
        mov.w     R13, &OSTCBCur        ; save as current
        mov.w     @R13, SP              ; restore new stack-ptr
        POPALL
        reti                            ; return from interrup

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
    .align  
WDT_ISR:                                ; wd timer ISR
        PUSHALL             
        bic.b    #0x01, IE1             ; disable wd-timer interrupt
        cmp.b    #0, &OSIntNesting      ; if (OSIntNesting == 0)  
        jne      WDT_ISR_1
        mov.w    &OSTCBCur, R13         ; save task-stack
        mov.w    SP, 0(R13)
        mov.w    &OSISRStkPtr, SP       ; load interrupt stack             
WDT_ISR_1:
        inc.b    &OSIntNesting          ; increment OSIntNesting
        bis.b    #0x01, IE1             ; enable wd-timer interrupt
        EINT                            ; enable general-interrupt to allow for interrupt nesting
        call     #OSTickISR
        DINT                            ; disable general-interrupt BEFORE calling OSIntExit()
        call     #OSIntExit
        cmp.b    #0, &OSIntNesting      ; if (OSIntNesting == 0) 
        jne      WDT_ISR_2
        mov.w    &OSTCBHighRdy, R13     ; restore task-stack SP
        mov.w    @R13, SP
WDT_ISR_2:
        POPALL
        reti                            ; return from interrupt

;****************************************************************
;                     SPECIAL INT-SAVE CALLS
;                   OS_STK_TYPE OSEnterCrit(void)
;                   void OSExitCrit(OS_STK_TYPE)
;****************************************************************
    .align  
OSEnterCrit:
        mov.w     SR, R15
        dint
        ret

    .align  
OSExitCrit:
        mov.w     R15, SR
        ret


;****************************************************************
;                WD TIMER INTERRUPT VECTOR ENTRY
;                  MSP430Fxxx Interrupt vector             
;****************************************************************
    .sect   ".int10"
    .short  WDT_ISR               ; interrupt vector. Watchdog/Timer, Timer mode
 
;****************************************************************

