/*************************************************************
*
* Description:
*	Philips-XA - Example program using Raisonance-IDE
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define	TASK_STK_SIZE   256

#define            XA_CRYSTAL_FREQ   (24000000UL)
#define            OS_TICKS_PER_SEC  100

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];

OS_MUX Mutex1;
OS_EVG Events1;

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

void TimerISR( void );

void main()
{
    WDCON  = 0xF0;      // Disable watchdog ...
    WFEED1 = 0xA5;      // ... Feed watchdog so new config. takes effect
    WFEED2 = 0x5A;
    BCR    = 0x16;      // WAIT disabled, 16-bit data bus, 20 address lines

    OS_Init();

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);

    OS_Start();
}

void AppTickInit(void)
{
    U08 scale;
    U32 prescaler;
    U16 reload;

    scale = (SCR >> 2) & 0x03;   // Find out what is the prescaler selection on the timers
    switch(scale) {
        case 0: prescaler = 4L;  // OSC/4
                break;
        case 1: prescaler = 16L; // OSC/16
                break;
        case 2: prescaler = 64L; // OSC/64
                break;
        default: while(1);       // error
                break;
    }
    reload = (U32)65536L - (XA_CRYSTAL_FREQ / ((U32)10 * prescaler));
    OS_ENTER_CRITICAL();
    TMOD &= 0xF0;             // Timer 0: Non-gated, Timer mode, 16-bit auto reload
    TH0   = reload >> 8;      // Count  register
    TL0   = reload & 0x00FF;
    RTH0  = reload >> 8;      // Reload register
    RTL0  = reload & 0x00FF;
    IPA0 &= 0x0F;             // Set interrupt priority for timer 0
    IPA0 |= 1 << 4;
    TCON |= 0x10;             // Enable timer 0 (i.e. timer 0 allowed to run)
    IEL  |= 0x02;             // Enable timer 0 interrupts
    OS_EXIT_CRITICAL();
}

void Task1(void *Tdata)
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    Tdata = Tdata;
    i=0;
    AppTickInit();
    ptr=NULL;
    while(1) {
        while(i<2500) i++;                              // no Task-switch to MUTEX
        i=0;
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(10);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(10);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}


void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // suspending
        if(!returnOk) {
            while(i<2500) i++;                          // no Taskswitch to Task1 or Task3
            i=0;
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(10);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

void Task3(void *Tdata)
{
    U16 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        OS_TimeDly(10);                                 // test timer-wakeup
        i++;
    }


}


