/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_PhilipsXA_c.c                    *
*        Philips-XA Specific Code (FAR MEMORY MODEL)       *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;
    U32   temp, dtemp;

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first ! Philips-XA Model   */
    temp             = (U32)task;                         /* Convert 'task' to 32-bit value       */ 
    *--stk           = (U16)( temp & 0xFFFFL);            /* Place lower 16 bits of 'task'        */
    *--stk           = (U16)((temp >> 16) & 0x00FFL);     /* Place upper  8 bits of 'task'        */
    *--stk           = (U16)0x6666;                       /* R6   = 0x6666                        */
    *--stk           = (U16)0x5555;                       /* R5   = 0x5555                        */
    *--stk           = (U16)0x4444;                       /* R4   = 0x4444                        */
    *--stk           = (U16)0x3333;                       /* R3   = 0x3333                        */
    *--stk           = (U16)0x2222;                       /* R2   = 0x2222                        */
    dtemp            = (U32)Tdata;                        /* Convert 'data' to 32-bit value       */
    *--stk           = (U16)((dtemp >> 16) & 0x00FFL);    /* R1:R0 = data (in registers R1:R0)    */
    *--stk           = (U16)( dtemp & 0xFFFFL);           /*                                      */
    *--stk           = (U16)0x0000;                       /* ES   = 0x00                          */
    *--stk           = (U16)0xFFFF;                       /* SSEL = 0xFF                          */
    *--stk           = (U16)0x0000;                       /* PSW                                  */
    *--stk           = (U16)((temp >> 16) & 0x00FFL);     /* PC(H), i.e. in reverse order.        */
    *--stk           = (U16)( temp & 0xFFFFL);            /* PC(L)                                */
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
