/***************************************************************
 *                       pC/OS 1.95c                           *
 *                      a FREE - RTOS                          *
 *                                                             *
 *                 based on the C/OS 1.00                     *
 *                   by  Jean J. Labrosse                      *
 *                                                             *
 *    changed by Frank Goetze  -  www.embedded-os.de           *
 ***************************************************************
 *                       OS_PIC24_a.s                          *
 *        PIC24FJ Harvard Architecture Specific Code           *
 *                      using MPLAB-X                          *
 **************************************************************/
    .global _OSStartHighRdy
    .global _OSCtxSw
    .global _OSIntCtxSw

    .extern SR
    .equiv  SRL, _SR
    .equiv  IPL3, 0x0008

    .equ __HAS_EDS__, 1       ; manual define for PIC24FJ256
    
    .text
/***************************************************************
 *                         marcos
 **************************************************************/
    .macro OS_REGS_SAVE
        push.d w0
        push.d w2
        push.d w4
        push.d w6
        push.d w8
        push.d w10
        push.d w12
        push   w14

        push TBLPAG
    .ifdef __HAS_EDS__
        push DSRPAG
        push DSWPAG
    .else
        push PSVPAG
    .endif
        push RCOUNT
        push SR
        push CORCON
    .endm

    .macro OS_REGS_RESTORE
        pop  CORCON  
        pop  SR
        pop  RCOUNT
    .ifdef __HAS_EDS__
        pop  DSWPAG
        pop  DSRPAG
    .else
        pop  PSVPAG
    .endif
        pop  TBLPAG

        pop    w14
        pop.d  w12
        pop.d  w10
        pop.d  w8
        pop.d  w6
        pop.d  w4
        pop.d  w2
        pop.d  w0
    .endm

/***************************************************************
 *                     START MULTITASKING
 *                  void OSStartHighRdy(void)
 **************************************************************/
_OSStartHighRdy:
        mov    _OSTCBHighRdy, w0
        mov    [w0], w1
	mov    [--w1], w0
	mov    w0, SPLIM
	mov    w1, w15
        OS_REGS_RESTORE
        retfie

/***************************************************************
 *         INITIATE A CONTEXT SWITCH (From task level)
 *                    void OSCtxSw(void)
 **************************************************************/
_OSCtxSw:
        mov.b  SRL, wreg
        sl     w0, #8, w0
        btsc   CORCON, #IPL3
        bset   w0, #7
        ior    w0, [--w15], w0
        mov    w0, [w15++]
        OS_REGS_SAVE
	mov    SPLIM, w0
	mov    w0, [w15++]
        mov    _OSTCBCur, w0
        mov    w15, [w0]
        mov    _OSTCBHighRdy, w0
        mov    w0, _OSTCBCur
        mov    [w0], w1
	mov    [--w1], w0
	mov    w0, SPLIM
	mov    w1, w15
        OS_REGS_RESTORE
        retfie

/***************************************************************
 *           PERFORM A CONTEXT SWITCH (From an ISR)
 *                   void OSIntCtxSw(void)
 **************************************************************/
_OSIntCtxSw:
        mov    _OSTCBHighRdy, w0
        mov    w0, _OSTCBCur
        mov    [w0], w1
	mov    [--w1], w0
	mov    w0, SPLIM
	mov    w1, w15
        OS_REGS_RESTORE
        retfie

;****************************************************************
;                        HANDLE TICK ISR
;                 Interrupt-Entry for OSTickISR()
;****************************************************************
__T1Interrupt:
        OS_REGS_SAVE
	mov    SPLIM, w0
	mov    w0, [w15++]
        mov    #_OSIntNesting, w0
        inc.b  [w0], [w0]
        dec.b  _OSIntNesting, wreg
        bra    nz, T1_Cont
        mov    _OSTCBCur, w0
        mov    w15, [w0]
T1_Cont:
        call   _OSIntEnter
        call   _OSTickISR
        call   _OSIntExit
	mov    [--w15], w0
	mov    w0, SPLIM
        OS_REGS_RESTORE
        retfie 

/**************************************************************/
