/*************************************************************
*
* Description:
*	PIC32MX - Example program using MPLAB-X
*
* Author[s]:
*	Frank Goetze
**************************************************************/

// PIC32MX360F512L (72MHz) / PIC32MX534F064H (80MHz) minimal Configuration Bit Settings
#pragma config FNOSC = PRIPLL, FPLLIDIV = DIV_2, FPLLODIV = DIV_1, FPBDIV = DIV_1
#ifdef __32MX360F512L__
 #pragma config FPLLMUL = MUL_18
 #define FOSC              72000000U
 #define PORTS_BIT_OUT     0x07
 #define PORTS_BIT_POS_LED PORTS_BIT_POS_1
#else
 #pragma config FPLLMUL = MUL_20
 #define FOSC              (80000000U / 10)     // ??? why "/10" on PIC32MX534F064H ???
 #define PORTS_BIT_OUT     0x40
 #define PORTS_BIT_POS_LED PORTS_BIT_POS_6
#endif

// config of the project - add following include-pathes to your project
// "<your install directory> \Microchip\xc32\v2.05\pic32-libs\pic32m-libs" 
// "<your install directory> \Microchip\harmony\v2_05_01\framework"

#include <xc.h>
#include <sys/attribs.h>

#include "../../inc/pC_OS.h"

#include <peripheral/peripheral.h>

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define CORE_TICK_DIV         (FOSC / OS_TIMER_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
OS_SEM LedSem;
OS_TMR LedTimer;
OS_TMR ROnceTimer1;
OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

void SysTick_init(void)
{
    __builtin_mtc0(11,0,(__builtin_mfc0(9,0) + CORE_TICK_DIV));
    __builtin_mtc0(9,0,0);
    PLIB_INT_SourceFlagClear(INT_ID_0,INT_SOURCE_TIMER_CORE);
    PLIB_INT_SourceEnable(INT_ID_0,INT_SOURCE_TIMER_CORE);
}

void SysTick_update(void)
{
    IFS0bits.CTIF = 0;
    PLIB_INT_SourceFlagClear(INT_ID_0,INT_SOURCE_TIMER_CORE);
    __builtin_mtc0(11,0,(__builtin_mfc0(9,0) + CORE_TICK_DIV));
}

void LED_init(void)
{
    PLIB_PORTS_OpenDrainEnable(PORTS_ID_0, PORT_CHANNEL_D, 0);
    PLIB_PORTS_Write(PORTS_ID_0, PORT_CHANNEL_D, 0);
    PLIB_PORTS_DirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_OUT);
    PLIB_PORTS_PinToggle(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_LED);    // toggle LED_Dx
}

void LED_toggle(void)
{
    PLIB_PORTS_PinToggle(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_LED);    // toggle LED_Dx
}

void SystemInit(void)
{
    PLIB_INT_VectorPrioritySet(INT_ID_0,INT_VECTOR_CT,INT_PRIORITY_LEVEL2);
    PLIB_INT_VectorPrioritySet(INT_ID_0,INT_VECTOR_CS0,INT_PRIORITY_LEVEL2);
    PLIB_INT_MultiVectorSelect(INT_ID_0);
    
    PLIB_INT_SourceFlagClear(INT_ID_0,INT_SOURCE_SOFTWARE_0);
    PLIB_INT_SourceEnable(INT_ID_0,INT_SOURCE_SOFTWARE_0);
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // PIC32 pC/OS ISR-code on top of the spezific vector-entry code
{
    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;

    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif
    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
