/*************************************************************
*
* Description:
*   SAMD21 - Example program using Keil / IAR
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#include "system_samd21.h"

#define SAMD21_TAU        0
#define SAMD21_SPARKFUN   1
#define SAMD21_15x15      2
#define SAMD21_STEMMA     3

#define SAMD21_TYPE       SAMD21_STEMMA


#define PORTA         0                       // user-LED definition
#define LED_GROUP     PORTA
#if (SAMD21_TYPE == SAMD21_TAU)               // -- TAU board
 #define LED_PORT     PORT_PA27
 #define LED_PIN      PIN_PA27
#endif
#if (SAMD21_TYPE == SAMD21_SPARKFUN)          // -- Sparkfun
 #define LED_PORT     PORT_PA17
 #define LED_PIN      PIN_PA17
#endif
#if (SAMD21_TYPE == SAMD21_15x15)             // -- SAMD15x15
 #define LED_PORT     PORT_PA09
 #define LED_PIN      PIN_PA09
#endif
#if (SAMD21_TYPE == SAMD21_STEMMA)            // -- STEMMA QT (WS2812B NeoPixel)
 #define LED_PORT     PORT_PA18
 #define LED_PIN      PIN_PA18
#endif

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 OS_ALIGN_ATTR pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif
OS_P        Pipe1;
OS_Q        Queue1;
U08         Queue1_d[16];
OS_SMQ      MQueue1;
OS_STK_TYPE MQueue_d[16];

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

#ifndef GCLK_CLKCTRL_ID_DFLL48_Val
 #define   GCLK_CLKCTRL_ID_DFLL48_Val   0x0ul
#endif
#define   GCLK_CLKCTRL_ID_DFLL48      (GCLK_CLKCTRL_ID_DFLL48_Val << GCLK_CLKCTRL_ID_Pos)
void SysTick_init(void)
{
    SYSCTRL_REGS->SYSCTRL_XOSC32K  = SYSCTRL_XOSC32K_ENABLE(1);
    SYSCTRL_REGS->SYSCTRL_XOSC32K |= SYSCTRL_XOSC32K_STARTUP(5) | SYSCTRL_XOSC32K_AAMPEN(1) | SYSCTRL_XOSC32K_EN32K(1) | SYSCTRL_XOSC32K_XTALEN(1) | SYSCTRL_XOSC32K_ENABLE(1);
    while((SYSCTRL_REGS->SYSCTRL_PCLKSR & SYSCTRL_PCLKSR_XOSC32KRDY_Msk) == 0) __NOP();
    GCLK_REGS->GCLK_GENDIV  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(1);
    GCLK_REGS->GCLK_GENCTRL = GCLK_GENCTRL_ID(1) | GCLK_GENCTRL_SRC_XOSC32K | GCLK_GENCTRL_GENEN(1);
    GCLK_REGS->GCLK_CLKCTRL = GCLK_CLKCTRL_GEN_GCLK1 | GCLK_CLKCTRL_CLKEN(1) | GCLK_CLKCTRL_ID(0) ;
    SYSCTRL_REGS->SYSCTRL_DFLLCTRL  = SYSCTRL_DFLLCTRL_ENABLE(1);
    SYSCTRL_REGS->SYSCTRL_DFLLMUL   = SYSCTRL_DFLLMUL_CSTEP(5) | SYSCTRL_DFLLMUL_FSTEP(10); 	
    SYSCTRL_REGS->SYSCTRL_DFLLMUL  |= SYSCTRL_DFLLMUL_MUL(1465);    // 1465 * 32768 is approx 48MHZ
    SYSCTRL_REGS->SYSCTRL_DFLLCTRL |= SYSCTRL_DFLLCTRL_MODE(1);
    while((SYSCTRL_REGS->SYSCTRL_PCLKSR & SYSCTRL_PCLKSR_DFLLLCKC_Msk) == 0) __NOP();
    NVMCTRL_REGS->NVMCTRL_CTRLB |= NVMCTRL_CTRLB_RWS(1);
    GCLK_REGS->GCLK_GENDIV  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(0);
    GCLK_REGS->GCLK_GENCTRL = GCLK_GENCTRL_GENEN(1) | GCLK_GENCTRL_SRC_DFLL48M | GCLK_GENCTRL_ID(0);
    GCLK_REGS->GCLK_CLKCTRL = GCLK_CLKCTRL_CLKEN(1) | GCLK_CLKCTRL_GEN_GCLK0 | GCLK_CLKCTRL_ID_DFLL48;
    // ---------------------
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
}

void SysTick_update(void)
{
}

void LED_init(void)
{
    PORT_REGS->GROUP[LED_GROUP].PORT_DIRSET = LED_PORT;
    PORT_REGS->GROUP[LED_GROUP].PORT_OUTSET = LED_PORT;
#if (SAMD21_TYPE == SAMD21_STEMMA)
    PORT_REGS->GROUP[LED_GROUP].PORT_DIRSET = PORT_PA15;
    PORT_REGS->GROUP[LED_GROUP].PORT_OUTSET = PORT_PA15;
#endif
}

extern void output_ws2812b(uint8_t * pbuf, uint32_t count);    // sends out 'count' bytes of data located at 'pbuf'.

void LED_toggle(void)
{
#if (SAMD21_TYPE == SAMD21_STEMMA)
	  U08 led[] = {0xFF, 0x00, 0x00 };
    output_ws2812b(led, 3);
#else
    PORT_REGS->GROUP[LED_GROUP].PORT_OUTTGL = LED_PORT;
#endif
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);

    OS_TaskCreate(Task1, NULL, &Stk1, TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, &Stk2, TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, &Stk3, TASK_STK_SIZE, 12);

    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical Cortex-M0 pC/OS ISR
{
    OS_IntEnter();

    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
    OS_IntExit();
}

void AppTickInit(void)
{                                                          // Configure the timer/system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i, tmq[10];
    void *t = (void *)tmq;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((void **)&ptr);

    returnOk=OS_PipeInit(&Pipe1, NULL, 0, 0);
    returnOk=OS_PipePost(&Pipe1, (U08 OS_HUGE *)"0123456789", 10, OS_SUSPEND);
    returnOk=OS_PipePost(&Pipe1, (U08 OS_HUGE *)"abcdefghijklmnopqrst", 20, OS_SUSPEND);
    returnOk=OS_PipeFrontPost(&Pipe1, (U08 OS_HUGE *)".,-+*", 5, OS_SUSPEND);

    returnOk=OS_SMQueueInit(&MQueue1, MQueue_d, 16);
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);
    returnOk=OS_SMQueuePost(&MQueue1, (void **)&ptr, OS_SUSPEND);
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 20);
    returnOk=OS_SMQueueFrontPost(&MQueue1, (void **)&ptr, OS_SUSPEND);
    returnOk=OS_SMQueuePost(&MQueue1, &t, OS_SUSPEND);

    returnOk=OS_QueueInit(&Queue1, Queue1_d, 16);
    returnOk=OS_QueuePost(&Queue1, 0x31, OS_SUSPEND);
    returnOk=OS_QueueFrontPost(&Queue1, 0x32, OS_SUSPEND);

    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk, db, msg[30], *mq;
    U32 i;
    U16 l;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            returnOk=OS_PipePend(&Pipe1, msg, &l, OS_NO_SUSP);

            returnOk=OS_SMQueuePend(&MQueue1, (void **)&mq, OS_NO_SUSP);
            if ((mq != NULL) && (!OS_MemVerifyPtr(mq)))
                returnOk=OS_MemFree((void **)&mq);

            returnOk=OS_QueuePend(&Queue1, &db, OS_NO_SUSP);

            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    Tdata = Tdata;

    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
