/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the µC/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                   OS_ICC_STM32H7xx.c                     *
*               Dual-Core ICC Specific Code                *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

#define OS_ICC_SIZE  (48 + sizeof(U16))   // bytes per message + sizeof(U16)
#define OS_ICC_DEEP   16                  // max messages in ICC AMP-Pipe

//volatile struct shared_data * const shared_ptr = (struct shared_data *)0x30040000;

/***********************************************************
* creating both pipes channels and management structs in   *
* shared memory for both cores                             *
*                                                          *
* using sections                                           *
*   static OS_ICC ipc_0                               __attribute__((section("shared_memory"))) = {0};
*   static OS_ICC ipc_1                               __attribute__((section("shared_memory"))) = {0};
*   static U08 icc_0_data[(OS_ICC_SIZE * OS_IPC_DEEP] __attribute__((section("shared_memory"))) = {0};
*   static U08 icc_1_data[(OS_ICC_SIZE * OS_IPC_DEEP] __attribute__((section("shared_memory"))) = {0};
*                                                          *
* using manaul adress settings                             *
* (offset calculated with padding space between)           *
* like STM32H745 using manual "AHB SRAM3 at 0x30040000" or "SRAM4 at 0x38000000"
*   static OS_ICC  *IccRxS = (OS_IPC *)0x30040000;    // icc_0_shared;
*   static OS_ICC  *IccTxS = (OS_IPC *)0x30040010;    // icc_1_shared;
*   static U08 *IccTxDataS =    (U08 *)0x30040100;    // icc_0_shared_data;
*   static U08 *IccTxDataS =    (U08 *)0x30040500;    // icc_1_shared_data;
*                                                          *
***********************************************************/

#ifdef CORE_CM7
 static OS_ICC  *IccRxS = (volatile OS_ICC *)0x38000000;
 static OS_ICC  *IccTxS = (volatile OS_ICC *)0x38000080;
 static U08 *IccRxDataS =    (U08 *)0x38000100;
 static U08 *IccTxDataS =    (U08 *)0x38000500;
 #define ICC_IRQHandler  CM4_SEV_IRQHandler
 #define ICC_IRQn        CM4_SEV_IRQn
#endif 
#ifdef CORE_CM4
 static OS_ICC  *IccRxS = (volatile OS_ICC *)0x38000080;
 static OS_ICC  *IccTxS = (volatile OS_ICC *)0x38000000;
 static U08 *IccRxDataS =    (U08 *)0x38000500;
 static U08 *IccTxDataS =    (U08 *)0x38000100;
 #define ICC_IRQHandler  CM7_SEV_IRQHandler
 #define ICC_IRQn        CM7_SEV_IRQn
#endif
 #define ICC_Event()     { __DSB(); __SEV(); }

void ICC_IRQEnable(void)
{
    NVIC_EnableIRQ(ICC_IRQn);
}

void ICC_IRQHandler(void)
{
    OS_IntEnter();
    OS_IccEventHandler();
    NVIC_ClearPendingIRQ(ICC_IRQn);
    OS_IntExit();
}

