;/*****************************************************************************/
;/* TMS570.S: Startup file for Texas Instruments TMS570 device series         */
;/*****************************************************************************/
;/*                w/o FPU support                                            */
;/*****************************************************************************/

        SECTION .intvec:CODE:ROOT(2)

        ;; Forward declaration of sections.
        SECTION IRQ_STACK:DATA:NOROOT(3)
        SECTION FIQ_STACK:DATA:NOROOT(3)
        SECTION CSTACK:DATA:NOROOT(3)
        
; The Startup code is executed after CPU Reset.

;----------------------- CODE --------------------------------------------------

;                PRESERVE8

; Area Definition and Entry Point
;  Startup Code must be linked first at Address at which it expects to run.

    PROGRAM   ?RESET
    SECTION   .intvec:CODE:ROOT(2)
    ;RSEG      ICODE:CODE:ROOT(2)
    CODE32    ; Always ARM mode after reset	

        EXTERN  OSirqISR
        EXTERN  OSfiqISR

; Exception Vectors
;  Mapped to Address 0.
;  Absolute addressing mode must be used.
;  Dummy Handlers are implemented as infinite loops which can be modified.
Vectors         B       Reset_Handler        
                B       Undef_Handler
                B       SWI_Handler
                B       PAbt_Handler
                B       DAbt_Handler
                B       .
                B       OSirqISR        ; pC/OS IRQ-entry
                B       OSfiqISR        ; pC/OS FIQ-entry


Undef_Handler   B       Undef_Handler
SWI_Handler     B       SWI_Handler
PAbt_Handler    B       PAbt_Handler
DAbt_Handler    B       DAbt_Handler

IRQ_Handler     B       IRQ_Handler
FIQ_Handler     B       FIQ_Handler

; Reset Handler

                EXPORT  Reset_Handler
Reset_Handler   
                MOV     r0,  lr        ; Initialization of R4 core registers
                MOV     r1,  #0x0000   ; necessary for lock-step device
                MOV     r2,  #0x0000
                MOV     r3,  #0x0000
                MOV     r4,  #0x0000
                MOV     r5,  #0x0000
                MOV     r6,  #0x0000
                MOV     r7,  #0x0000
                MOV     r8,  #0x0000
                MOV     r9,  #0x0000
                MOV     r10, #0x0000
                MOV     r11, #0x0000
                MOV     r12, #0x0000
                MOV     r13, r12  ;To avoid warning mov r13, #0x0000
                MRS     r1, cpsr
                MSR     spsr_cxsf, r1 

                BL      .+4            ; Initialization of return stack
                BL      .+4
                BL      .+4
                BL      .+4

; Setup Stack for each mode ----------------------------------------------------
#define MODE_MSK 0x1F            ; Bit mask for mode bits in CPSR

#define USR_MODE 0x10            ; User mode
#define FIQ_MODE 0x11            ; Fast Interrupt Request mode
#define IRQ_MODE 0x12            ; Interrupt Request mode
#define SVC_MODE 0x13            ; Supervisor mode
#define ABT_MODE 0x17            ; Abort mode
#define UND_MODE 0x1B            ; Undefined Instruction mode
#define SYS_MODE 0x1F            ; System mode 

        MRS     r0, cpsr                ; Original PSR value

        ;; Set up the interrupt stack pointer.
        BIC     r0, r0, #MODE_MSK       ; Clear the mode bits
        ORR     r0, r0, #IRQ_MODE       ; Set IRQ mode bits
        MSR     cpsr_c, r0              ; Change the mode
        LDR     r1, =SFE(IRQ_STACK)     ; End of IRQ_STACK
        BIC     sp,r1,#0x7              ; Make sure SP is 8 aligned

        ;; Set up the fast interrupt stack pointer.
        BIC     r0, r0, #MODE_MSK       ; Clear the mode bits
        ORR     r0, r0, #FIQ_MODE       ; Set FIR mode bits
        MSR     cpsr_c, r0              ; Change the mode
        LDR     r1, =SFE(FIQ_STACK)     ; End of FIQ_STACK
        BIC     sp,r1,#0x7              ; Make sure SP is 8 aligned

        ;; Set up the normal stack pointer.
        BIC     r0 ,r0, #MODE_MSK       ; Clear the mode bits
        ORR     r0 ,r0, #SYS_MODE       ; Set System mode bits
        MSR     cpsr_c, r0              ; Change the mode
        LDR     r1, =SFE(CSTACK)        ; End of CSTACK
        BIC     sp,r1,#0x7              ; Make sure SP is 8 aligned 

; Enter the C code -------------------------------------------------------------

;                IMPORT  systemInit
;                LDR     R0, =systemInit
;                BLX     R0

                IMPORT  main
                LDR     R0, =main
                BX      R0


                END
