/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                       OS_V850_c.c                        *
*                Renesas V850 Specific Code                *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;
#if USE_FPU
    U08          i;
#endif

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
    *--stk   = (OS_STK_TYPE) pstk;                        /* task stack pointer                   */
    *--stk   = (OS_STK_TYPE) 0x00000000;                  /* Task PSW = Interrupts disabled       */
    *--stk   = (OS_STK_TYPE) task;                        /* Task's PC code                       */
    *--stk   = (OS_STK_TYPE) 0x31313131;                  /* r31 Link pointer                     */
    *--stk   = (OS_STK_TYPE) 0x30303030;                  /* r30                                  */
    *--stk   = (OS_STK_TYPE) 0x29292929;                  /* r29                                  */
    *--stk   = (OS_STK_TYPE) 0x28282828;                  /* r28 Frame pointer/permanent          */
    *--stk   = (OS_STK_TYPE) 0x27272727;                  /* r27                                  */
    *--stk   = (OS_STK_TYPE) 0x26262626;                  /* r26                                  */
    *--stk   = (OS_STK_TYPE) 0x25252525;                  /* r25                                  */
    *--stk   = (OS_STK_TYPE) 0x24242424;                  /* r24                                  */
    *--stk   = (OS_STK_TYPE) 0x23232323;                  /* r23                                  */
    *--stk   = (OS_STK_TYPE) 0x22222222;                  /* r22                                  */
    *--stk   = (OS_STK_TYPE) 0x21212121;                  /* r21                                  */
    *--stk   = (OS_STK_TYPE) 0x20202020;                  /* r20                                  */
    *--stk   = (OS_STK_TYPE) 0x19191919;                  /* r19                                  */
    *--stk   = (OS_STK_TYPE) 0x18181818;                  /* r18                                  */
    *--stk   = (OS_STK_TYPE) 0x17171717;                  /* r17                                  */
    *--stk   = (OS_STK_TYPE) 0x16161616;                  /* r16                                  */
    *--stk   = (OS_STK_TYPE) 0x15151515;                  /* r15                                  */
    *--stk   = (OS_STK_TYPE) 0x14141414;                  /* r14                                  */
    *--stk   = (OS_STK_TYPE) 0x13131313;                  /* r13                                  */
    *--stk   = (OS_STK_TYPE) 0x12121212;                  /* r12                                  */
    *--stk   = (OS_STK_TYPE) 0x11111111;                  /* r11                                  */
    *--stk   = (OS_STK_TYPE) 0x10101010;                  /* r10 Function return value/temporary  */
    *--stk   = (OS_STK_TYPE) 0x09090909;                  /* r9 Parameters                        */
    *--stk   = (OS_STK_TYPE) 0x08080808;                  /* r8 Parameters                        */
    *--stk   = (OS_STK_TYPE) 0x07070707;                  /* r7 Parameters                        */
    *--stk   = (OS_STK_TYPE) 0x06060606;                  /* r6 Parameters                        */
    *--stk   = (OS_STK_TYPE) 0x05050505;                  /* r5 Temporary or Rosda base           */
    *--stk   = (OS_STK_TYPE) 0x02020202;                  /* r2 Temporary                         */
    *--stk   = (OS_STK_TYPE) Tdata;                       /* r1 data / Address generation in compiler and assembler */
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}

/**********************************************************/
