/***********************************************************
*                       pC/OS 1.94b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_Rabbit_c.c                       *
*                   Z-180 Specific code                    *
*                   BANKED MEMORY MODEL                    *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    SETUP THE TASK STACK
***********************************************************
*/
OS_STK_TYPE *OSTaskInitStk(void (OS_FAR *task)(void *dptr), void *Tdata, void **pstk, U16 stksize) KC51_REENTRANT
{
    OS_STK_TYPE *stk;
    U16          task_bank;
    U16          task_offset;

    // ---- HW specific ----
    stk      = &((OS_STK_TYPE *)*pstk)[stksize];          /* decrement first !                    */
    task_bank        = (U16)((U32)task >> 16);
    task_bank        = (task_bank << 8) + task_bank;
    task_offset      = (U16)((U32)task & 0xFFFFL);    
    *--stk           = (U16)task_bank;                    /* Save bank repeated in upper and lower register        */
    *--stk           = (U16)task_offset;                  /* Save task address                    */
    *--stk           = (U16)task_offset;                  /* Saved PC from ISR (Simulated)        */
    *--stk           = (U16)0x1100;                       /* AF   Primary registers               */
    *--stk           = (U16)0x2222;                       /* BC                                   */
    *--stk           = (U16)Tdata;                        /* DE = Tdata is passed in DE           */
    *--stk           = (U16)0x4444;                       /* HL                                   */
    *--stk           = (U16)0x5555;                       /* AF   Alternate registers             */
    *--stk           = (U16)0x6666;                       /* BC                                   */
    *--stk           = (U16)0x7777;                       /* DE                                   */
    *--stk           = (U16)0x8888;                       /* HL                                   */
    *--stk           = (U16)0x9999;                       /* IX                                   */
    *--stk           = (U16)0xAAAA;                       /* IY                                   */
    *--stk           = (U16)task_bank;                    /* Save bank (repeated in upper and lower register) */
    // ---------------------
    // pstk is the descending end-of-stack and must not be updated
    return stk;
}
