/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_mem.c                         *
*                     MEMORY SERVICE                       *
***********************************************************/

#if  OS_MEM_EN
/*
************************************************************
*                GLOBAL AND LOCAL VARIABLES                *
************************************************************
*/
    OS_MEM OS_HUGE *OSMFirstPtr;

#if OS_MEM_CHECK_EN
    U32             OSMFullSize;
/*
************************************************************
*                 CHECK MEMORY (internal)                  *
************************************************************
*/
U08 OSMemoryCheck(OS_MEM OS_HUGE *curr_MEM)
{
    if ((curr_MEM < OSMFirstPtr) || (curr_MEM > &OSMFirstPtr[OSMFullSize-sizeof(OS_MEM)]) ||
        ((curr_MEM->OSMPrev) && (curr_MEM->OSMPrev->OSMNext != curr_MEM)) ||
        ((curr_MEM->OSMNext) && ((curr_MEM->OSMNext->OSMPrev != curr_MEM) || 
         (curr_MEM->OSMLen > OSMFullSize))))
        return 1;
    return 0;
}
#endif
/*
************************************************************
*                    INITIALIZE MEMORY                     *
************************************************************
*/
U08 OS_MemoryInit(void OS_HUGE *mp, U32 size) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32          mpc;
    
    if ((!size) || (!mp))
        return (OS_MEM_ERR);
    OS_ENTER_CRITICAL();
    OSLockNesting++;
    OS_EXIT_CRITICAL();
    mpc = (U32)mp;
    if ((mpc % sizeof(OS_STK_TYPE)) != 0) {
        mpc  += (sizeof(OS_STK_TYPE) - (mpc % sizeof(OS_STK_TYPE)));
        size -= (sizeof(OS_STK_TYPE) - (mpc % sizeof(OS_STK_TYPE)));
    }    
    OSMFirstPtr = (OS_MEM OS_HUGE *)mpc;
    OSMFirstPtr->OSMNext  = (OS_MEM OS_HUGE *)((U08 OS_HUGE *)(mpc) + size - sizeof(OS_MEM));
    OSMFirstPtr->OSMPrev  = NULL;
    OSMFirstPtr->OSMLen   = 0;
#if OS_TASK_DESTROY_EN
    OSMFirstPtr->OSMId    = (OS_STK_TYPE)-1;
#endif
    OSMFirstPtr->OSMNext->OSMNext  = NULL;
    OSMFirstPtr->OSMNext->OSMPrev  = OSMFirstPtr;
    OSMFirstPtr->OSMNext->OSMLen   = 0;
#if OS_TASK_DESTROY_EN
    OSMFirstPtr->OSMNext->OSMId    = (OS_STK_TYPE)-1;
#endif
#if OS_MEM_CHECK_EN
    OSMFullSize = size;
#endif
    OS_ENTER_CRITICAL();
    if (OSLockNesting != 0)
        OSLockNesting--;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*               ALLOCATE MEMORY (internal)                 *
************************************************************
*/
U08 OSMemAlloc(void OS_HUGE *(*MemPtr), U32 size, U08 TCBid) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE     OS_sr;
#endif
    OS_MEM OS_HUGE *current_MEM;
    OS_MEM OS_HUGE *new_MEM;

    if ((!size) || (size > ((0x0UL-1) - sizeof(OS_MEM))))
        return (OS_MEM_ERR);
    OS_ENTER_CRITICAL();
    OSLockNesting++;
    OS_EXIT_CRITICAL();
    size += sizeof(OS_MEM);
    if ((size % sizeof(OS_STK_TYPE)) != 0)
        size += (sizeof(OS_STK_TYPE) - (size % sizeof(OS_STK_TYPE)));
    current_MEM = OSMFirstPtr;
    while (1) {
#if OS_MEM_CHECK_EN
        if (OSMemoryCheck(current_MEM) != 0)
            OSMemChkError();
#endif
        if ((((U08 OS_HUGE *)current_MEM->OSMNext)-
        ((U08 OS_HUGE *)current_MEM) - current_MEM->OSMLen) >= size)
            break;
        current_MEM = current_MEM->OSMNext;
        if (!current_MEM->OSMNext) {
            OS_ENTER_CRITICAL();
            if (OSLockNesting != 0)
                OSLockNesting--;
            OS_EXIT_CRITICAL();
            return (OS_MEM_OVF);
        }
    }
    if (!current_MEM->OSMLen) {
        current_MEM->OSMLen = size;
#if OS_TASK_DESTROY_EN
        current_MEM->OSMId  = TCBid;
#endif
        *MemPtr = (U08 OS_HUGE *)(current_MEM) + sizeof(OS_MEM);
        OS_ENTER_CRITICAL();
        if (OSLockNesting != 0)
            OSLockNesting--;
        OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    }
    new_MEM = (OS_MEM OS_HUGE *)((U08 OS_HUGE *)(current_MEM) + current_MEM->OSMLen);
    new_MEM->OSMNext     = current_MEM->OSMNext;
    new_MEM->OSMPrev     = current_MEM;
    current_MEM->OSMNext = new_MEM;
    if (new_MEM->OSMNext)
        new_MEM->OSMNext->OSMPrev = new_MEM;
    new_MEM->OSMLen      = size;
#if OS_TASK_DESTROY_EN
    new_MEM->OSMId       = TCBid;
#endif
    *MemPtr = (U08 OS_HUGE *)(new_MEM) + sizeof(OS_MEM);
    OS_ENTER_CRITICAL();
    if (OSLockNesting != 0)
        OSLockNesting--;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                    ALLOCATE MEMORY                       *
************************************************************
*/
U08 OS_MemAlloc(void OS_HUGE *(*MemPtr), U32 size) KC51_REENTRANT
{
    return (OSMemAlloc(MemPtr, size, OSTCBCur->OSTCBId));
}
/*
************************************************************
*                 FREE MEMORY  (internal)                  *
************************************************************
*/
U08 OSMemFree(void OS_HUGE *MemPtr, U08 TCBid) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE     OS_sr;
#endif
    OS_MEM OS_HUGE *curr_MEM;

    if (!MemPtr)
        return (OS_MEM_ERR);
    OS_ENTER_CRITICAL();
    OSLockNesting++;
    OS_EXIT_CRITICAL();
    curr_MEM = (OS_MEM OS_HUGE *)((U08 OS_HUGE *)(MemPtr) - sizeof(OS_MEM));
#if OS_MEM_CHECK_EN
    if ((OSMemoryCheck(curr_MEM) != 0) || (curr_MEM->OSMId != TCBid))
        OSMemChkError();
#endif
    if ((curr_MEM == OSMFirstPtr) || (curr_MEM == curr_MEM->OSMPrev->OSMNext)) {
        if (curr_MEM->OSMPrev) {
            curr_MEM->OSMPrev->OSMNext = curr_MEM->OSMNext;
            if (curr_MEM->OSMNext)
                curr_MEM->OSMNext->OSMPrev = curr_MEM->OSMPrev;
        } else
            curr_MEM->OSMLen = 0;
        OS_ENTER_CRITICAL();
        if (OSLockNesting != 0)
            OSLockNesting--;
        OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    }
    OS_ENTER_CRITICAL();
    if (OSLockNesting != 0)
        OSLockNesting--;
    OS_EXIT_CRITICAL();
    return (OS_MEM_ERR);
}
/*
************************************************************
*                      FREE MEMORY                         *
************************************************************
*/
U08 OS_MemFree(void OS_HUGE *(*MemPtr)) KC51_REENTRANT
{
    U08 ret;

    ret = OSMemFree(*MemPtr, OSTCBCur->OSTCBId);
    if (ret == OS_NO_ERR) *MemPtr = NULL;
    return ret;
}
/*
************************************************************
*                       GET FREE-SIZE                      *
************************************************************
*/
U32 OS_MemFreeSize(void) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE     OS_sr;
#endif
    OS_MEM OS_HUGE *curr_MEM;
    U32             fsize, len;

    OS_ENTER_CRITICAL();
    OSLockNesting++;
    OS_EXIT_CRITICAL();
    curr_MEM = OSMFirstPtr;
    fsize = 0;
    while (1) {
        len = ((U08 OS_HUGE *)curr_MEM->OSMNext) - ((U08 OS_HUGE *)curr_MEM) - curr_MEM->OSMLen;
        if(len > sizeof(OS_MEM)) {
            fsize += (len - sizeof(OS_MEM));
        }
        curr_MEM = curr_MEM->OSMNext;
        if(!(curr_MEM->OSMNext)) break;
    }
    OS_ENTER_CRITICAL();
    if (OSLockNesting != 0)
        OSLockNesting--;
    OS_EXIT_CRITICAL();
    return(fsize);
}
/*
************************************************************
*                  VERIFY MEMORY POINTER                   *
************************************************************
*/
U08 OS_MemVerifyPtr(void OS_HUGE *MemPtr) KC51_REENTRANT
{
    if (((OS_MEM OS_HUGE *)MemPtr < OSMFirstPtr) || ((OS_MEM OS_HUGE *)MemPtr > &OSMFirstPtr[OSMFullSize/sizeof(OS_MEM)]))
        return (OS_MEM_ERR);
    return (OS_NO_ERR);
}

#if OS_TASK_DESTROY_EN
/*
************************************************************
*                CLEAN UP A USER (internal)                *
************************************************************
*/
U08 OSMemCleanUp(U08 userId)
{
    OS_MEM OS_HUGE *curr_MEM;

    curr_MEM = OSMFirstPtr;
    while (curr_MEM != NULL) {
        if ((curr_MEM->OSMId == userId) && (curr_MEM->OSMLen != 0)) {
            if (OSMemFree((void OS_HUGE *)((U08 OS_HUGE *)(curr_MEM) + sizeof(OS_MEM)), userId) != OS_NO_ERR)
                return (OS_MEM_ERR);
        }
        curr_MEM = curr_MEM->OSMNext;
    }
    return (OS_NO_ERR);
}
#endif /* OS_TASK_DESTROY_EN */
#endif /* OS_MEM_EN */

/************************* END ****************************/


