/***********************************************************
*                       pC/OS 1.95c                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                     pC_OS_mutex.c                        *
*                     MUTEX SERVICE                        *
***********************************************************/

#if OS_MUX_EN
/*
************************************************************
*                    CREATE A MUTEX                        *
************************************************************
*/
U08 OS_MutexCreate(OS_MUX *pmux, U08 p) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08 i;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)NULL) {
        OSTCBPrioTbl[p]   = (OS_TCB *)1;
        pmux->OSMuxPrio   = p;
        pmux->OSMuxOrg    = 0x00;
        pmux->OSMuxGrp    = 0x00;
        for (i = 0; i < OS_PRIO_TBL; i++)
            pmux->OSMuxTbl[i] = 0x00;
        OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}
/*
************************************************************
*                     PEND ON MUTEX                        *
************************************************************
*/
U08  OS_MutexPend(OS_MUX *pmux, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity;

    OS_ENTER_CRITICAL();
    if (pmux->OSMuxPrio > OSTCBCur->OSTCBPrio) {
        OS_EXIT_CRITICAL();
        return (OS_MUX_ERR);
    }
    if (pmux->OSMuxOrg != 0) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_MUX_NOACC);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = pmux;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_MUX;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        pmux->OSMuxTbl[y] |= bitx;
        pmux->OSMuxGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if ((OSTCBCur->OSTCBStat & OS_STAT_MUX) && (pmux->OSMuxOrg != 0)) {
            if ((U08)(pmux->OSMuxTbl[y] &= ~(bitx)) == 0)
                pmux->OSMuxGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (pmux->OSMuxOrg != 0) {
            OS_EXIT_CRITICAL();
            return (OS_MUX_NOACC);
        }
    }
    pmux->OSMuxOrg = OSTCBCur->OSTCBPrio;
    x = pmux->OSMuxOrg;
    y = pmux->OSMuxPrio;
    if ((OSRdyTbl[x >> 3] &= ~(OSMapTbl[x & 0x07])) == 0)
        OSRdyGrp &= ~(OSMapTbl[x >> 3]);
    OSRdyGrp            |= OSMapTbl[y >> 3];
    OSRdyTbl[y >> 3]    |= OSMapTbl[y & 0x07];
    OSTCBCur->OSTCBPrio  = y;
    OSTCBPrioTbl[y]      = OSTCBCur;
    OSTCBPrioTbl[x]      = (OS_TCB *)1;
#if OS_TASK_DESTROY_EN
    OSTCBCur->OSTCBmux   = pmux;
#endif
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                     POST TO A MUTEX                      *
************************************************************
*/
U08  OS_MutexPost(OS_MUX *pmux) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pmux->OSMuxPrio == OSTCBCur->OSTCBPrio) {
        x = pmux->OSMuxPrio;
        y = pmux->OSMuxOrg;
        pmux->OSMuxOrg = 0x00;
        if ((OSRdyTbl[x >> 3] &= ~(OSMapTbl[x & 0x07])) == 0)
            OSRdyGrp &= ~(OSMapTbl[x >> 3]);
        OSRdyGrp            |= OSMapTbl[y >> 3];
        OSRdyTbl[y >> 3]    |= OSMapTbl[y & 0x07];
        OSTCBCur->OSTCBPrio  = y;
        OSTCBPrioTbl[y]      = OSTCBCur;
        OSTCBPrioTbl[x]      = (OS_TCB *)1;
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBmux   = NULL;
#endif
        if (pmux->OSMuxGrp != 0) {
            y    = OSUnMapTbl[pmux->OSMuxGrp];
            x    = OSUnMapTbl[pmux->OSMuxTbl[y]];
            bity = OSMapTbl[y];
            bitx = OSMapTbl[x];
            if ((U08)(pmux->OSMuxTbl[y] &= ~(bitx)) == 0) {
                pmux->OSMuxGrp &= ~(bity);
            }
            p                           = (y << 3) + x;
            OSTCBPrioTbl[p]->OSTCBDly   =  0;
            OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_MUX);
            OSRdyGrp                   |=  bity;
            OSRdyTbl[y]                |=  bitx;
            OS_EXIT_CRITICAL();
            OSSched();
        } else
            OS_EXIT_CRITICAL();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_MUX_ERR);
}
/*
************************************************************
*             ABBORT HIGHEST PENDER ON A MUTEX             *
************************************************************
*/
U08 OS_MutexPendAbbort(OS_MUX *pmux) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pmux->OSMuxGrp != 0) {
        y    = OSUnMapTbl[pmux->OSMuxGrp];
        x    = OSUnMapTbl[pmux->OSMuxTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pmux->OSMuxTbl[y] &= ~(bitx)) == 0) {
            pmux->OSMuxGrp &= ~(bity);
        }
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSRdyGrp                   |=  bity;
        OSRdyTbl[y]                |=  bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}

#if OS_TASK_DESTROY_EN
/*
************************************************************
*            DELETE A PENDING TASK FROM A MUTEX            *
************************************************************
*/
void OSMutexDestroyTask(OS_MUX *pmux, U08 prio)
{
    U08          x, y, bitx, bity;

    y      = prio >> 3;
    x      = prio & 0x07;
    bity   = OSMapTbl[y];
    bitx   = OSMapTbl[x];
    if ((U08)(pmux->OSMuxTbl[y] &= ~(bitx)) == 0)
        pmux->OSMuxGrp &= ~(bity);
    OSTCBPrioTbl[prio]->OSTCBDly  =  0;
    OSRdyGrp                     |=  bity;
    OSRdyTbl[y]                  |=  bitx;
}
/*
************************************************************
*         FREE A MUTEX FROM A HOLDING ACTIVE TASK          *
************************************************************
*/
U08 OSMutexFreeTask(OS_MUX *pmux, U08 prio)
{
    U08          orgp, x, y, bitx, bity, p;

    orgp = 0xFF;
    if (pmux->OSMuxPrio == prio) {
        x        = pmux->OSMuxPrio;
        orgp = y = pmux->OSMuxOrg;
        pmux->OSMuxOrg = 0x00;
        if ((OSRdyTbl[x >> 3] &= ~(OSMapTbl[x & 0x07])) == 0)
            OSRdyGrp &= ~(OSMapTbl[x >> 3]);
        OSRdyGrp            |= OSMapTbl[y >> 3];
        OSRdyTbl[y >> 3]    |= OSMapTbl[y & 0x07];
        OSTCBPrioTbl[y]      = OSTCBPrioTbl[x];
        OSTCBPrioTbl[x]      = (OS_TCB *)1;
        OSTCBPrioTbl[y]->OSTCBPrio  = y;
        OSTCBPrioTbl[y]->OSTCBmux   = NULL;
        if (pmux->OSMuxGrp != 0) {
            y    = OSUnMapTbl[pmux->OSMuxGrp];
            x    = OSUnMapTbl[pmux->OSMuxTbl[y]];
            bity = OSMapTbl[y];
            bitx = OSMapTbl[x];
            if ((U08)(pmux->OSMuxTbl[y] &= ~(bitx)) == 0)
                pmux->OSMuxGrp &= ~(bity);
            p                           = (y << 3) + x;
            OSTCBPrioTbl[p]->OSTCBDly   =  0;
            OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_MUX);
            OSRdyGrp                   |=  bity;
            OSRdyTbl[y]                |=  bitx;
        }
        OSMutexReInitResource(pmux, OSTCBPrioTbl[orgp]->OSTCBId);
    }
    return (orgp);
}
#endif /* OS_TASK_DESTROY_EN */
#endif /* OS_MUX_EN */

/************************* END ****************************/


